/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.core;

import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.lang.Validate;
import org.apache.rat.pd.heuristic.IHeuristicChecker;
import org.apache.rat.pd.util.FileManipulator;

/**
 * This class can parse command line arguments and retrieve arguments from file.
 * If some arguments are not provided, it set default values to them. Only
 * -codebase argument is required. Arguments passed to constructor are parsed
 * and values are set to proper class member. After that configuration file is
 * parsed. If path to configuration file is provided by command line argument,
 * we try to parse that configuration file. Otherwise, default configuration
 * "config.conf" file is parsed. After this two steps if any of members is not
 * initialized, we initialize them with default values. Finally,
 * report,codebase,limit,engines, heuristics and allArgumentsCorrect are
 * initialized. If allArgumentsCorrect is true, we can use parsed values.
 * Otherwise, we can just print usage.
 * 
 * @author maka
 */
public class PdCommandLine {
    private static final int MINIMAL_WAITING_TIME = 1000;
    // possible report formats
    public static final String HTML_MODE = "html";
    public static final String TEXT_MODE = "txt";
    public static final String XML_MODE = "xml";

    private static final String VERBOSE_OPT_DESC = "verbose description of current actions";
    private static final String LANGUAGE_OPT_DESC = "name of a language";
    private static final String HEURISTICS_OPT_DESC = "list of heuristics";
    private static final String ENGINES_OPT_DESC = "list of engines";
    private static final String CONFIG_OPT_DESC = "name of the config file";
    private static final String LIMIT_OPT_DESC = "integer from 1 to 100,1 is almost brute force and 100 is heuristic";
    private static final String CODEBASE_OPT_DESC = "name of source code directory";
    private static final String REPORT_OPT_DESC = "name of the report file;\nextension of file determines report format:"
            + "\n\t.xml  - report will be xml document"
            + "\n\t.html - report will be html document"
            + "\n\t.txt  - report will be txt document" + "\nDefault is report.html.";
    private static final String HELP_OPT_DESC = "print this message";
    private static final String WAITING_TIME_OPT_DESC = "integer which represents waiting time between two queries in milliseconds";
    private static final String VERBOSE_OPT_SHORT = "v";
    private static final String LANGUAGE_OPT_SHORT = "lang";
    private static final String HEURISTIC_OPT_SHORT = "heur";
    private static final String ENGINES_OPT_SHORT = "e";
    private static final String CONFIG_OPT_SHORT = "c";
    private static final String LIMIT_OPT_SHORT = "l";
    private static final String CODEBASE_OPT_SHORT = "cb";
    private static final String REPORT_OPT_SHORT = "r";
    private static final String HELP_OPT_SHORT = "h";
    private static final String WAITING_TIME_SHORT = "w";
    public static final int DEFAULT_LIMIT = 10;
    public static final String DEFAULT_HEURISTIC = "BruteForceHeuristicChecker";
    public static final String DEFAULT_ENGINE = "GoogleCodeSearchParser";
    public static final String DEFAULT_REPORT = "report.html";
    public static final int DEFAULT_WAITING_TIME = 2000;
    private static final String DEFAULT_HEURISTIC_MSG = "Heuristic is not set.In use is default checker: ";
    private static final String DEFAULT_ENGINE_MSG = "Engines is not set.In use is default engine: ";
    private static final String DEFAULT_LIMIT_MSG = "Limit is not set.In use is default limit: ";
    private static final String DEFAULT_RAPORT_MSG = "Report is not set.In use is default report: ";
    private static final String DEFAULT_WAITING_TIME_MSG = "Waiting time is not set. In use is default waiting time: ";
    private static final String DEFAULT_LANGUAGE_MSG = "Progremming language is not set.In use is default language: ";
    private static final String VERBOSE_OPT_LONG = "verbose";
    private static final String LANGUAGE_OPT_LONG = "language";
    private static final String HEURISTICS_OPT_LONG = "heuristics";
    private static final String ENGINES_OPT_LONG = "engines";
    private static final String CONFIG_OPT_LONG = "config";
    private static final String LIMIT_OPT_LONG = "limit";
    private static final String CODEBASE_OPT_LONG = "codebase";
    private static final String REPORT_OPT_LONG = "report";
    private static final String HELP_OPT_LONG = "help";
    private static final String WAITING_TIME_LONG = "wait";

    // this are properties which can be parsed from
    // command line or from configuration file
    private Options options = new Options();
    private String report;
    private String codebase;
    /**
     * Limit is user-defined constants 1-100 and it is used to measure how
     * aggressively will be used search engines if we put there 10 =little usage
     * of Internet (search engines) if we put there 90 =almost bruteforce
     * checking will be run
     */
    private int limit;
    private List<String> engines = new ArrayList<String>();
    private String config = "config.conf";
    private List<String> heuristics = new ArrayList<String>();
    private boolean allArgumentsCorrect = true;
    private String language;
    private boolean verbose = false;
    private int waitingTime;

    private final PrintStream out;
    // available options
    private Option helpOption = new Option(HELP_OPT_SHORT, HELP_OPT_LONG, false, HELP_OPT_DESC);

    private Option reportOption = OptionBuilder.hasArg(true).withArgName("file").withDescription(REPORT_OPT_DESC)
            .withLongOpt(REPORT_OPT_LONG).create(REPORT_OPT_SHORT);

    private Option codebaseOption = OptionBuilder.hasArg(true).withArgName("dir").withDescription(CODEBASE_OPT_DESC)
            .withLongOpt(CODEBASE_OPT_LONG).create(CODEBASE_OPT_SHORT);

    private Option limitOption = OptionBuilder.hasArg(true).withArgName("int").withDescription(LIMIT_OPT_DESC)
            .withLongOpt(LIMIT_OPT_LONG).create(LIMIT_OPT_SHORT);

    private Option configOption = OptionBuilder.hasArg(true).withArgName("file").withDescription(CONFIG_OPT_DESC)
            .withLongOpt(CONFIG_OPT_LONG).create(CONFIG_OPT_SHORT);

    private Option enginesOption = OptionBuilder.withValueSeparator(',').hasArgs(Option.UNLIMITED_VALUES).withArgName(
            "names").withDescription(ENGINES_OPT_DESC).withLongOpt(ENGINES_OPT_LONG).create(ENGINES_OPT_SHORT);

    private Option heuristicsOption = OptionBuilder.withValueSeparator(',').hasArgs(Option.UNLIMITED_VALUES)
            .withArgName("names").withDescription(HEURISTICS_OPT_DESC).withLongOpt(HEURISTICS_OPT_LONG).create(
                    HEURISTIC_OPT_SHORT);

    private Option languageOption = OptionBuilder.hasArg(true).withArgName("name").withDescription(LANGUAGE_OPT_DESC)
            .withLongOpt(LANGUAGE_OPT_LONG).create(LANGUAGE_OPT_SHORT);

    private Option verboseOption = OptionBuilder.hasArg(false).withDescription(VERBOSE_OPT_DESC).withLongOpt(
            VERBOSE_OPT_LONG).create(VERBOSE_OPT_SHORT);

    private Option waitingTimeOption = OptionBuilder.hasArg(true).withArgName("int").withDescription(
            WAITING_TIME_OPT_DESC).withLongOpt(WAITING_TIME_LONG).create(WAITING_TIME_SHORT);

    /**
     * Arguments passed to constructor are parsed and values are set to proper
     * class member. After that configuration file is parsed. If path to
     * configuration file is provided by command line argument, we try to parse
     * that configuration file. Otherwise, default configuration "config.conf"
     * file is parsed. After this two steps if any of members is not
     * initialized, we initialize them with default values.
     * 
     * @param args command line arguments
     * @param out output stream for displaying current information
     */
    public PdCommandLine(String[] args, PrintStream out) {
        this.out = out;
        initializeOptions();
        // we parse command line
        try {
            parseArguments(args);
            // from command line we get configuration file path
            // or use default one, and then parse config file for more options
            parseConfigFile(this.config);
            // only if it is codebase set from command line or configuration
            // file we continue
            Validate.notNull(this.codebase, "Codebase must be provided.");
            // if any option is not configured after this, we set it to
            // default value
            configureUninitializedValues();
        } catch (Exception e) {
            // oops, something went wrong
            this.out.println("Parsing failed.  Reason: " + e.getMessage());
            // if any error happened, we can say that not all arguments are
            // correct
            allArgumentsCorrect = false;
        }

        // TODO for debugging, delete me
        this.out.println("codebase    : " + this.codebase);
        this.out.println("limit       : " + this.limit);
        this.out.println("engines     : " + this.engines);
        this.out.println("heuristic   : " + this.heuristics);
        this.out.println("config      : " + this.config);
        this.out.println("report      : " + this.report);
        this.out.println("format      : " + this.getReportFormat());
        this.out.println("language    : " + this.language);
        this.out.println("verbose     : " + this.verbose);
        this.out.println("waitingTime : " + this.waitingTime);
    }

    /**
     * Parse arguments and gets parameters from them.
     * 
     * @param args command line arguments
     * @throws ParseException
     * @throws NumberFormatException
     */
    private void parseArguments(String[] args) throws ParseException, NumberFormatException {
        CommandLineParser parser = new GnuParser();
        CommandLine line = parser.parse(options, args);
        parseCommandLineArguments(line);
    }

    /**
     * Sets default values to uninitialized properties.
     */
    private void configureUninitializedValues() {
        if (this.report == null) {
            this.out.println(DEFAULT_RAPORT_MSG + DEFAULT_REPORT);
            this.report = DEFAULT_REPORT;
        }
        if (this.limit == 0) {
            this.out.println(DEFAULT_LIMIT_MSG + DEFAULT_LIMIT);
            this.limit = DEFAULT_LIMIT;
        }
        if (this.engines.isEmpty()) {
            this.out.println(DEFAULT_ENGINE_MSG + DEFAULT_ENGINE);
            this.engines.add(DEFAULT_ENGINE);
        }
        if (this.heuristics.isEmpty()) {
            this.out.println(DEFAULT_HEURISTIC_MSG + DEFAULT_HEURISTIC);
            this.heuristics.add(DEFAULT_HEURISTIC);
        }
        if (this.waitingTime <= 0) {
            this.out.println(DEFAULT_WAITING_TIME_MSG + DEFAULT_WAITING_TIME);
            this.waitingTime = DEFAULT_WAITING_TIME;
        }
        if (this.language == null) {
            this.out.println(DEFAULT_LANGUAGE_MSG + IHeuristicChecker.ALL_LANGUAGES);
            this.language = IHeuristicChecker.ALL_LANGUAGES;
        }
    }

    /**
     * Prints message how to use PdCommandLine.
     */
    public void printUsage() {
        StringBuffer footer = new StringBuffer("\n");

        footer.append("Example: java -jar apache-rat-pd.jar -cb c:\\src -r c:\\report.txt -lang java -v\n");
        footer.append("NOTE:\n");
        footer.append("This tool shows report in console and save it to file\n");
        footer.append("Remember - this is very basic tool!!!\n");

        // automatically generate the help statement
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp("java -jar apache-rat-pd.jar [options]", "Options:", options, footer.toString(), false);
    }

    /**
     * Initializes available options.
     */
    private void initializeOptions() {
        options.addOption(helpOption);
        options.addOption(reportOption);
        options.addOption(codebaseOption);
        options.addOption(limitOption);
        options.addOption(configOption);
        options.addOption(enginesOption);
        options.addOption(heuristicsOption);
        options.addOption(languageOption);
        options.addOption(verboseOption);
        options.addOption(waitingTimeOption);
    }

    /**
     * Parse command line and sets values to properties.
     * <p>
     * If some property is invalid an exception is thrown.
     * 
     * @param line CommandLine
     * @throws NumberFormatException
     * @throws IllegalArgumentException
     */
    private void parseCommandLineArguments(CommandLine line) throws NumberFormatException, IllegalArgumentException {

        if (line.hasOption(CODEBASE_OPT_LONG)) {
            this.codebase = line.getOptionValue(CODEBASE_OPT_LONG);
        }
        if (line.hasOption(REPORT_OPT_LONG)) {
            this.report = line.getOptionValue(REPORT_OPT_LONG);
        }
        if (line.hasOption(CONFIG_OPT_LONG)) {
            this.config = line.getOptionValue(CONFIG_OPT_LONG);
        }
        if (line.hasOption(ENGINES_OPT_LONG)) {
            String[] temp = line.getOptionValues(ENGINES_OPT_LONG);
            for (int i = 0; i < temp.length; i++) {
                engines.add(temp[i]);
            }
        }
        if (line.hasOption(HEURISTICS_OPT_LONG)) {
            String[] temp = line.getOptionValues(HEURISTICS_OPT_LONG);
            for (int i = 0; i < temp.length; i++) {
                heuristics.add(temp[i]);
            }
        }

        if (line.hasOption(LANGUAGE_OPT_LONG)) {
            this.language = line.getOptionValue(LANGUAGE_OPT_LONG);
        }

        if (line.hasOption(HELP_OPT_LONG)) {
            allArgumentsCorrect = false;
        }
        if (line.hasOption(LIMIT_OPT_LONG)) {
            int temp = Integer.valueOf(line.getOptionValue(LIMIT_OPT_LONG));
            // this line will throw an exception if number is not in specified
            // interval
            Validate.isTrue(temp > 0 && temp <= 100, "Limit must be in interval [1,100] inclusive");
            this.limit = temp;
        }
        if (line.hasOption(VERBOSE_OPT_LONG)) {
            this.verbose = true;
        }
        if (line.hasOption(WAITING_TIME_LONG)) {
            int temp = Integer.valueOf(line.getOptionValue(WAITING_TIME_LONG));
            // this line will throw an exception if number is not greater than
            // zero
            Validate.isTrue(temp >= MINIMAL_WAITING_TIME, "Waiting time must be greater or equal than 1000ms");
            this.waitingTime = temp;
        }
    }

    /**
     * @return limit property
     */
    public int getLimit() {
        return this.limit;
    }

    /**
     * @return verbose flag
     */
    public boolean isVerbose() {
        return this.verbose;
    }

    /**
     * @return filename of the report
     */
    public String getReport() {
        return this.report;
    }

    /**
     * @return format of report, PdCommandLine.HTML_MODE, PdCommandLine.XML_MODE
     *         or PdCommandLine.TEXT_MODE
     */
    public String getReportFormat() {
        String toRet = getReport() != null ? getReport().replaceFirst(".*\\.", "") : "";
        // htm and html are valid extensions for html files
        if (toRet.equalsIgnoreCase("html") || toRet.equalsIgnoreCase("htm")) {
            toRet = PdCommandLine.HTML_MODE;
        }
        if (toRet.equalsIgnoreCase("xml")) {
            toRet = PdCommandLine.XML_MODE;
        }
        return toRet;
    }

    /**
     * @return path to code base
     */
    public String getCodebase() {
        return this.codebase;
    }

    /**
     * @return list of engine names
     */
    public List<String> getEngines() {
        return this.engines;
    }

    /**
     * @return list of heuristic names
     */
    public List<String> getHeuristics() {
        return this.heuristics;
    }

    /**
     * @return programming language name
     */
    public String getLanguage() {
        return this.language;
    }

    /**
     * @return waiting time between two queries
     */
    public int getWaitingTime() {
        return this.waitingTime;
    }

    /**
     * @return true if all arguments are parsed properly
     */
    public boolean isAllArgumentsCorrect() {
        return allArgumentsCorrect;
    }

    /**
     * Read configuration file and parse it. We assert that configuration file
     * structure is like command line - in each line is defined an argument and
     * value. Available arguments are same as command line options.
     * 
     * @param config path to configuration file
     * @throws NumberFormatException
     * @throws ParseException
     */
    private void parseConfigFile(String config) throws NumberFormatException, ParseException {
        this.out.println("Trying to read configuration options from: " + config);
        String[] args = null;
        try {
            String content = FileManipulator.readFile(new FileReader(config));
            content = content.replaceAll("\n", " ");
            args = content.split(" ");
        } catch (IOException e) {
            this.out.println("Error parsing configuration options from: " + config);
        }
        parseArguments(args);
    }

}
