/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.apache.rat.DirectoryWalker;
import org.apache.rat.pd.engines.ISearchEngine;
import org.apache.rat.pd.engines.google.GoogleCodeSearchParser;
import org.apache.rat.pd.heuristic.BruteForceHeuristicChecker;
import org.apache.rat.pd.heuristic.IHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.ActionScriptCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.CCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.CPPCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.CSharpCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.CobolCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.ColdFusionCommentsheuristicChecker;
import org.apache.rat.pd.heuristic.comment.DelphiCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.FortranCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.HTMLCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.JavaCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.JavaScriptCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.LispCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.PHPCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.PascalCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.PerlCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.PythonCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.RubyCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.SQLCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.ShellCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.comment.VisualBasicCommentHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.ActionScriptFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.CFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.CPPFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.CSharpFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.DelphiFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.JavaFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.JavaScriptFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.PHPFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.functions.PascalFunctionHeuristicChecker;
import org.apache.rat.pd.heuristic.misspellings.Dictionary;
import org.apache.rat.pd.heuristic.misspellings.IDictionary;
import org.apache.rat.pd.heuristic.misspellings.MisspellingsHeuristicChecker;
import org.apache.rat.pd.report.HtmlReportGenerator;
import org.apache.rat.pd.report.Report;
import org.apache.rat.pd.report.TxtReportGenerator;
import org.apache.rat.pd.report.XmlReportGenerator;
import org.apache.rat.pd.util.FileManipulator;
import org.apache.rat.report.RatReportFailedException;

/**
 * This simple demo tool checks is source code is cut&pasted from somewhere by
 * searching on Google code search, Koders code search.... Tool is still
 * unfinished with very basic usability..... more on:
 * http://wiki.apache.org/general/MarijaSljivovic/SoC2009ApacheRatProposal
 * 
 * @author maka
 */
public class PlagiarismDetector {

    // stream for printing current information
    private PrintStream out;

    /**
     * @param args
     * @throws RatReportFailedException
     * @throws IOException
     */
    public static void main(String[] args) throws RatReportFailedException, IOException {
        // instance of PlagiarismDetector
        PlagiarismDetector pd = new PlagiarismDetector();

        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        if (!pdCommandLine.isAllArgumentsCorrect()) {
            pdCommandLine.printUsage();
            System.exit(1);
        }
        // stream for printing current information
        pd.out = pd.getProperPrintStream(pdCommandLine);
        // we add all rules we are interested in
        final List<IHeuristicChecker> algorithmsForChecking = pd.configureHeuristicCheckers(pdCommandLine);
        // making one or more parsers
        final List<ISearchEngine> searchEngines = pd.configureSearchEngines(pdCommandLine, pd.getLanguage(
                pdCommandLine, algorithmsForChecking));

        // we check whole directory
        final DirectoryWalker walker = new DirectoryWalker(new File(pdCommandLine.getCodebase()));
        // report document
        Report reportDocument = pd.configureReport(pdCommandLine);

        // analyzer is sliding-window checker
        final SourceCodeAnalyser analyser = new SourceCodeAnalyser(searchEngines, algorithmsForChecking,
                reportDocument, pd.out);

        final PlagiarismDetectorReport plagiarismDetectorReport = new PlagiarismDetectorReport(analyser);

        walker.run(plagiarismDetectorReport);
        // Saving generated report
        pd.saveReport(pdCommandLine.getReport(), reportDocument.getReport());
    }

    /**
     * Decide which format of report will be used.
     * 
     * @param pdCommandLine command line object to read report format from
     * @return which format of report will be used
     */
    private Report configureReport(PdCommandLine pdCommandLine) {
        Report reportDocument;
        if (pdCommandLine.getReportFormat().equals(PdCommandLine.TEXT_MODE)) {
            reportDocument = new TxtReportGenerator();
        } else if (pdCommandLine.getReportFormat().equals(PdCommandLine.XML_MODE)) {
            reportDocument = new XmlReportGenerator();
        } else {
            reportDocument = new HtmlReportGenerator();
        }
        return reportDocument;
    }

    /**
     * This method can create objects defined from command line search engine
     * arguments
     * 
     * @param pdCommandLine command line object to read engines from
     * @param language name of programming language
     * @return list of chosen search engine parsers
     */
    private List<ISearchEngine> configureSearchEngines(PdCommandLine pdCommandLine, String language) {
        List<ISearchEngine> toRet = new ArrayList<ISearchEngine>();
        for (String searchEngine : pdCommandLine.getEngines()) {
            if (searchEngine.equals("GoogleCodeSearchParser")) {
                toRet.add(new GoogleCodeSearchParser(language, pdCommandLine.getWaitingTime(), out));
            }
            // add more parsers here(koders and krugle)
        }
        return toRet;
    }

    /**
     * This method can create objects defined from command line heuristic
     * checker arguments
     * 
     * @param pdCommandLine command line object to read engines from
     * @return list of chosen heuristic checkers
     * @throws IOException
     */
    private List<IHeuristicChecker> configureHeuristicCheckers(PdCommandLine pdCommandLine) throws IOException {
        List<IHeuristicChecker> toRet = new ArrayList<IHeuristicChecker>();

        for (String heuristicChecker : pdCommandLine.getHeuristics()) {
            if (heuristicChecker.equals("BruteForceHeuristicChecker")) {
                toRet.add(new BruteForceHeuristicChecker(pdCommandLine.getLanguage()));
            }
            // ///////////////////////////////////////////////////
            // Comment heuristic checkers
            // //////////////////////////////////////////////////
            else if (heuristicChecker.equals("JavaCommentHeuristicChecker")) {
                toRet.add(new JavaCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("PascalCommentHeuristicChecker")) {
                toRet.add(new PascalCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CCommentHeuristicChecker")) {
                toRet.add(new CCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CobolCommentHeuristicChecker")) {
                toRet.add(new CobolCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("ColdFusionCommentsheuristicChecker")) {
                toRet.add(new ColdFusionCommentsheuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("DelphiCommentHeuristicChecker")) {
                toRet.add(new DelphiCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("FortranCommentHeuristicChecker")) {
                toRet.add(new FortranCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("HTMLCommentHeuristicChecker")) {
                toRet.add(new HTMLCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("LispCommentHeuristicChecker")) {
                toRet.add(new LispCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("PHPCommentHeuristicChecker")) {
                toRet.add(new PHPCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("PythonCommentHeuristicChecker")) {
                toRet.add(new PythonCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("RubyCommentHeuristicChecker")) {
                toRet.add(new RubyCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("SQLCommentHeuristicChecker")) {
                toRet.add(new SQLCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("VisualBasicCommentHeuristicChecker")) {
                toRet.add(new VisualBasicCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("ActionScriptCommentHeuristicChecker")) {
                toRet.add(new ActionScriptCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CPPCommentHeuristicChecker")) {
                toRet.add(new CPPCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CSharpCommentHeuristicChecker")) {
                toRet.add(new CSharpCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("JavaScriptCommentHeuristicChecker")) {
                toRet.add(new JavaScriptCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("PerlCommentHeuristicChecker")) {
                toRet.add(new PerlCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("ShellCommentHeuristicChecker")) {
                toRet.add(new ShellCommentHeuristicChecker(pdCommandLine.getLimit(), out));
            }
            // ///////////////////////////////////////////////////
            // Functions heuristic checkers
            // //////////////////////////////////////////////////
            else if (heuristicChecker.equals("ActionScriptFunctionHeuristicChecker")) {
                toRet.add(new ActionScriptFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CFunctionHeuristicChecker")) {
                toRet.add(new CFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CPPFunctionHeuristicChecker")) {
                toRet.add(new CPPFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("CSharpFunctionHeuristicChecker")) {
                toRet.add(new CSharpFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("DelphiFunctionHeuristicChecker")) {
                toRet.add(new DelphiFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("JavaFunctionHeuristicChecker")) {
                toRet.add(new JavaFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("JavaScriptFunctionHeuristicChecker")) {
                toRet.add(new JavaScriptFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("PascalFunctionHeuristicChecker")) {
                toRet.add(new PascalFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            } else if (heuristicChecker.equals("PHPFunctionHeuristicChecker")) {
                toRet.add(new PHPFunctionHeuristicChecker(pdCommandLine.getLimit(), out));
            }
            // ///////////////////////////////////////////////////
            // Misspellings heuristic checkers
            // //////////////////////////////////////////////////
            else if (heuristicChecker.equals("MisspellingsHeuristicChecker")) {
                InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("misspellings/english.dict");
                IDictionary dictionary = new Dictionary(is);
                toRet.add(new MisspellingsHeuristicChecker(dictionary, pdCommandLine.getLanguage(), out));
            }

            else {
                System.err.println("Cannot find propper class for " + heuristicChecker);
            }
        }
        return toRet;
    }

    /**
     * @param reportName file name
     * @param reportContent content of report file
     */
    private void saveReport(String raportName, String reportContent) {
        try {
            FileManipulator.saveFile(raportName, reportContent);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @param algorithmsForChecking list of heuristic checkers
     * @return Language of source code. It depends by selected heuristic
     *         checkers.
     */
    private String getLanguageFromHeuristic(List<IHeuristicChecker> algorithmsForChecking) {
        String toRet = IHeuristicChecker.ALL_LANGUAGES;
        Set<String> langs = new TreeSet<String>();
        for (IHeuristicChecker heuristicChecker : algorithmsForChecking) {
            langs.add(heuristicChecker.getLanguage());
        }
        if (langs.size() == 1) {
            toRet = langs.iterator().next();
            out.println("Chosen language is: " + toRet);
        }
        return toRet;
    }

    /**
     * Decides which programming language will be used.
     * 
     * @param commandLine commandLine object to read language information from
     * @param algorithmsForChecking list of heuristic checkers
     * @return programming language name
     */
    private String getLanguage(PdCommandLine commandLine, List<IHeuristicChecker> algorithmsForChecking) {
        String toRet = IHeuristicChecker.ALL_LANGUAGES;
        if (!commandLine.getLanguage().equals(IHeuristicChecker.ALL_LANGUAGES)) {
            toRet = commandLine.getLanguage();
        } else {
            toRet = getLanguageFromHeuristic(algorithmsForChecking);
        }
        return toRet;
    }

    /**
     * If it is wanted to print current information in console, getVerbose will
     * return true. Otherwise, an dummy all information are printed in dummy
     * output stream.
     * 
     * @param commandLine commandLine object to read verbose information from
     * @return chosen output stream. If -v is not specified, all information
     *         will not be shown
     */
    private PrintStream getProperPrintStream(PdCommandLine pdCommandLine) {
        return pdCommandLine.isVerbose() ? System.out : new PrintStream(new OutputStream() {
            @Override
            public void write(int b) {
            } // Do nothing when printing is requested.
        });
    }
}
