/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.core;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;

import org.apache.rat.document.IDocument;
import org.apache.rat.document.IDocumentAnalyser;
import org.apache.rat.document.RatDocumentAnalysisException;
import org.apache.rat.pd.engines.ISearchEngine;
import org.apache.rat.pd.engines.SearchResult;
import org.apache.rat.pd.heuristic.HeuristicCheckerResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;
import org.apache.rat.pd.report.Report;
import org.apache.rat.pd.report.ReportEntry;
import org.apache.rat.pd.util.FileManipulator;

/**
 * This is implementation of the sliding window algorithm
 * 
 * @author maka
 */
public class SourceCodeAnalyser implements IDocumentAnalyser {

    private static String STRING_DELIMETER_REGEX = " +";
    private final List<ISearchEngine> searchEngines;
    private final List<IHeuristicChecker> algorithmsForChecking;
    private final Report reportDocument;

    private final PrintStream out;
    private PauseListener pauseListener;
    private volatile boolean paused = false;
    private int progressMessageLength;

    /**
     * @param searchEngines list of chosen search engine parsers
     * @param algorithmsForChecking list of chosen heuristic algorithms
     * @param reportDocument chosen report
     * @param out printStream where current information are to be printed
     */
    public SourceCodeAnalyser(List<ISearchEngine> searchEngines, List<IHeuristicChecker> algorithmsForChecking,
            Report reportDocument, PrintStream out) {
        this.searchEngines = searchEngines;
        this.algorithmsForChecking = algorithmsForChecking;
        this.reportDocument = reportDocument;
        this.out = out;

        initializeKeyListener();
    }

    /**
     * initialize key listener for pause/resume
     */
    private void initializeKeyListener() {
        this.pauseListener = new PauseListener() {
            @Override
            public void onPause() {
                System.out.println("apache-rat-pd is paused...");
                System.out.println("to resume application, pres Enter");
                SourceCodeAnalyser.this.paused = true;
            }

            @Override
            public void onResume() {
                System.out.println("apache-rat-pd resumed...");
                SourceCodeAnalyser.this.paused = false;
            }

        };
        // to ensure that this process will end together with the program
        pauseListener.setDaemon(true);
        pauseListener.start();

    }

    /**
     * this method will make algorithm to sleep until flag paused == true
     */
    private void pauseIfNeeded() {
        while (paused)
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.apache.rat.document.IDocumentAnalyser#analyse(org.apache.rat.document
     * .IDocument)
     */
    @Override
    public void analyse(IDocument document) throws RatDocumentAnalysisException {
        try {

            System.out.println("\n");
            String code = FileManipulator.readFile(document.reader());

            checkOneFile(searchEngines, code, algorithmsForChecking, document.getName());
        } catch (IOException e) {
            System.err.println("Source file not found: " + document.getName());
        }
    }



    /**
     * Implementation of sliding-window algorithm.
     * 
     * @param searchEngines list of chosen search engine parsers
     * @param code code part to be checked
     * @param heuristicCheckers list of chosen heuristic checkers
     * @param fileName name of file we are currently checking
     */
    public void checkOneFile(List<ISearchEngine> searchEngines, String code, List<IHeuristicChecker> heuristicCheckers,
            String fileName) {

        String[] tokens = tokeniseString(code);
        ReportEntry raportEntry = null;

        int i = 0;
        int j = 0;

        while (i < tokens.length) {
            while (j < tokens.length) {
                // this method will make application to sleep if it is needed
                pauseIfNeeded();

                StringBuffer toCheck = combineTokens(tokens, i, j);
                j++;
                printProgress(tokens.length, j, fileName);

                HeuristicCheckerResult heurResult = isCheckOnInternetNeaded(heuristicCheckers, toCheck.toString());
                if (heurResult.isCheckOnInternetNeaded()) {

                    ReportEntry lastEntry = searchOnInternet(searchEngines, heurResult.getCodeSuggestedToBeChecked(),
                            fileName);
                    if (lastEntry != null) {
                        raportEntry = lastEntry;
                    }
                    if (!heurResult.isShouldStretch() || lastEntry == null) {
                        break;
                    }
                }
            }
            i = j;
            if (raportEntry != null) {
                printReport(raportEntry);
                reportDocument.addReportEntry(raportEntry);
                raportEntry = null;
            }
        }
    }

    // TODO move this to report package....
    /**
     * print current state into console
     * 
     * @param reportEntry
     */
    private void printReport(ReportEntry reportEntry) {
        out.println("chunk of code copied:");
        out.println("--------------------------------------------");
        out.println(reportEntry.getCode());
        out.println("--------------------------------------------");
    }

    /**
     * Search for code in all engines. If code is found, a reportEntry will be
     * returned. Otherwise, null will be returned.
     * 
     * @param searchEngines list of chosen search engine parsers
     * @param code code to be checked
     * @param fileName source file name
     * @return a report entry
     */
    private ReportEntry searchOnInternet(List<ISearchEngine> searchEngines, String code, String fileName) {
        ReportEntry toRet = null;
        List<SearchResult> found = new ArrayList<SearchResult>();
        for (ISearchEngine searchEngine : searchEngines) {
            if (searchEngine.isCodeFound(code.toString())) {
                found.addAll(searchEngine.getSearchResults());
            }
        }
        if (!found.isEmpty()) {
            toRet = new ReportEntry(found, code, fileName);
        }
        return toRet;
    }

    /**
     * @param heuristicCheckers list of chosen heuristic checkers
     * @param toCheck code to be checked
     * @return heuristicCheckerResult with proper information for sliding window
     *         algorithm
     */
    private HeuristicCheckerResult isCheckOnInternetNeaded(List<IHeuristicChecker> heuristicCheckers, String toCheck) {

        HeuristicCheckerResult toRet = new HeuristicCheckerResult(false, true, toCheck, null);
        for (IHeuristicChecker checker : heuristicCheckers) {
            toRet = checker.checkByHeuristic(toCheck);
            if (toRet.isShouldStretch()) {
                break;
            }
        }

        return toRet;
    }

    /**
     * Append tokens from start position until end.
     * 
     * @param tokens list of tokens to combine
     * @param start index of start position
     * @param end index of end position
     * @return appended tokens
     */
    private StringBuffer combineTokens(String[] tokens, int start, int end) {

        StringBuffer sb = new StringBuffer();
        for (int k = start; k < end; k++) {
            sb.append(tokens[k]);
            sb.append(" ");
        }
        sb.append(tokens[end]);
        return sb;
    }

    /**
     * extract tokens
     * 
     * @param code to be tokenised
     * @return list of words
     */
    private String[] tokeniseString(String code) {
        code = code.replaceAll("\\n", "\n ");
        String[] tokens = code.split(STRING_DELIMETER_REGEX);
        // this simple tokeniser returns array {""} when "" is tokenised
        // I must avoid that behavior
        if (tokens.length == 1 && tokens[0].equals(""))
            tokens = new String[0];
        return tokens;
    }

    /**
     * This method just prints in console information about current progress.
     *<p>
     * Example:Analyzing file : c:\HelloWorld.java Progress: 2/200 (1%)
     * 
     * @param whole number of all tokens we iterate
     * @param current current position of iteration
     * @param fileName file name of source file
     */
    private void printProgress(int whole, int current, String fileName) {
        // clear previous state
        for (int i = 0; i < progressMessageLength; i++) {
            System.out.print("\b");
        }

        String message = "Analyzing file: " + fileName + " Progress: " + current + "/" + whole + " ("
                + (current * 100 / whole) + "%)";
        this.progressMessageLength = message.length();
        System.out.print(message);
    }
}
