/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.engines;


/**
 * This class is here to hide retry and timeout logic. This logic protects
 * application from exceptions caused by network problems or late response from
 * server. This is separate class because all ISearchEngine implementations will
 * use it.
 * 
 * @author maka
 */
public class RetryManager {
    private final Managable searchengine;
    private final int numberOfRetry;
    private final int waitingTime;

    /**
     * @param searchengine engine to check code
     * @param numberOfRetry maximum number of retry before give up
     * @param timeout time between two requests
     */
    public RetryManager(Managable searchengine, int numberOfRetry, int waitingTime) {
        this.searchengine = searchengine;
        this.waitingTime = waitingTime;
        this.numberOfRetry = numberOfRetry;
    }

    /**
     * Retrieve information is code plagiarised. This method can several times
     * query SearchEngine for this information. If an exception happened, it
     * will wait before next retry.
     * 
     * @param possibleCopyAndPastedCode code currently being checked
     * @return information is code plagiarised
     */
    public boolean isCodeFound(String possibleCopyAndPastedCode) {
        boolean toRet = false;
        for (int i = 0; i < numberOfRetry; i++) {
            try {
                toRet = searchengine.queryEngine(possibleCopyAndPastedCode);
                break;
            } catch (Exception e) {
                // just print number of retry
                // Possible too long query is often catch here.
                System.out.println("Search Engine error. - " + e.getMessage() + ". retry..." + i);
            } finally {
                try {
                    Thread.currentThread().sleep(waitingTime);
                } catch (InterruptedException e) {
                    // just print message
                    System.err.println("sleap interupted...");
                }
            }
        }
        return toRet;
    }

}
