/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.engines.google;

import java.io.IOException;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.apache.rat.pd.engines.ISearchEngine;
import org.apache.rat.pd.engines.Managable;
import org.apache.rat.pd.engines.RetryManager;
import org.apache.rat.pd.engines.SearchResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;

import com.google.gdata.client.codesearch.CodeSearchService;
import com.google.gdata.data.codesearch.CodeSearchEntry;
import com.google.gdata.data.codesearch.CodeSearchFeed;
import com.google.gdata.data.codesearch.Match;
import com.google.gdata.util.ServiceException;

/**
 * This class is for communication between search engine and this program.
 * 
 * @author maka
 */
public class GoogleCodeSearchParser implements ISearchEngine, Managable {

    /**
     * This constant describe minimum of distance between two strings to decide
     * that they are similar It is from interval (0-100). Larger number means
     * more restrictive matching
     */
    public static final int SIMILARITY = 50;
    // numberOfRetry maximum number of retry before give up
    public static final int NUMBER_OF_RETRY = 5;
    // 2324 is too long
    public static final int CODESEARCH_QUERY_MAX_LENGTH = 2000;

    private static final String CODESEARCH_FEEDS_URL = "http://www.google.com/codesearch/feeds/search?";

    public static final String RESULT_NUMBER = "20";

    /**
     * This class is there to estimate matching of results returned by
     * GoogleCodeSearch with real code. It will read CodeSearchFeed and analyze
     * matching between Google results and actual code part. Maybe it will be
     * better if GoogleCodeSearch can return file in which matched query, but it
     * is not possible from gdata-codesearch api. //TODO Think about using third
     * party api for getting code from GoogleCodeSearch site. HTMLUnit can do
     * that.
     * 
     * @author maka
     */
    private class Postprocessor {

        private final int similarityFactor;

        public Postprocessor(int similarityFactor) {
            this.similarityFactor = similarityFactor;
        }

        /**
         * Decide are strings similar using various algorithms for string
         * similarity. If similarity is found based on any type of checking,
         * results will be true.
         * 
         * @param searchResults list of results for our query
         * @param possibleCopyAndPastedCode code currently being checked
         * @return information is there any match found
         */
        public boolean isMatchFound(List<SearchResult> searchResults, String possibleCopyAndPastedCode) {
            boolean toRet = false;
            String[] lines = possibleCopyAndPastedCode.split("\n *");

            // iterate ower results
            for (SearchResult searchResult : searchResults) {
                // TODO if matches are duplicated, there is unnecessary checking
                for (String match : searchResult.getMatchedCode()) {

                    for (String line : lines) {
                        if (isMatchingByDistance(match, line) || this.isMatchingByRegex(match, line)) {
                            toRet = true;
                            break;
                        }
                    }
                }
            }
            return toRet;
        }

        /**
         * Make decision are strings similar by calculation of string distance
         * 
         * @param source source code
         * @param toMatch code for comparation
         * @return information are code parts similar based on
         *         LevenshteinDistance algorithm
         */
        boolean isMatchingByDistance(String source, String toMatch) {

            int maxLength = toMatch.length() > source.length() ? toMatch.length() : source.length();
            int distance = StringUtils.getLevenshteinDistance(toMatch, source);
            boolean toRet = (maxLength > 0) && (similarityFactor < 100 - (distance * 100) / maxLength);
            // TODO think about lines with only one character. }
            // for example!!!
            return toRet;
        }

        /**
         * Make decision are strings similar by matching by regular expression
         * 
         * @param source source code
         * @param toMatch code for comparation
         * @return information are code parts similar based on similarity by
         *         regular expression
         */
        boolean isMatchingByRegex(String source, String toMatch) {

            Pattern p = Pattern.compile(regexGenerator.stringToRegex(toMatch));

            Matcher m = p.matcher(source);
            return m.find();
        }
    }

    private Postprocessor postprocessor = new Postprocessor(SIMILARITY);
    private CodeSearchService codesearchService = new CodeSearchService("apache-rat-pd");
    private List<SearchResult> searchResults = new ArrayList<SearchResult>();
    private RegexGenerator regexGenerator = new MultilineRegexGenerator();

    private final RetryManager retryManager;
    private final String language;
    private final PrintStream out;

    public GoogleCodeSearchParser(String language, int waitingTime, PrintStream out) {
        this.language = language;
        this.out = out;
        this.retryManager = new RetryManager(this, NUMBER_OF_RETRY, waitingTime);
    }

    private String getLanguageQuery() {
        String toRet = "";
        if (!language.equals(IHeuristicChecker.ALL_LANGUAGES)) {
            toRet = "as_lang=" + this.language + "&";
        }
        return toRet;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.rat.pd.engines.ISearchEngine#getSearchResults()
     */
    @Override
    public List<SearchResult> getSearchResults() {
        return searchResults;
    }

    /**
     * @param possibleCopyAndPastedCode code currently being checked
     * @return proper URL for gdata-codesearch API
     * @throws MalformedURLException
     */
    @SuppressWarnings("deprecation")
    // FIXME URLEncoding is system dependent now
    private URL createUrl(String possibleCopyAndPastedCode) throws MalformedURLException {
        URL toRet = new URL(CODESEARCH_FEEDS_URL + getLanguageQuery() + "q="
                + URLEncoder.encode(regexGenerator.stringToRegex(possibleCopyAndPastedCode)) + "&max-results="
                + RESULT_NUMBER);

        return toRet;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.apache.rat.pd.engines.ISearchEngine#isCodeFound(java.lang.String)
     */
    @Override
    public boolean isCodeFound(String possibleCopyAndPastedCode) {
        searchResults.clear();
        return retryManager.isCodeFound(possibleCopyAndPastedCode);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.rat.pd.engines.Managable#gueryEngine(java.lang.String)
     */
    @Override
    public boolean queryEngine(String possibleCopyAndPastedCode) throws IOException, ServiceException {
        boolean toReturn = false;

        String[] lines = possibleCopyAndPastedCode.split("\n *");
        // lines must be less then 1024 characters
        // this is bug-fix of bug
        // http://code.google.com/p/apache-rat-pd/source/detail?r=51#
        List<String> splitedLines = generateLinesWithProperLength(lines, CODESEARCH_QUERY_MAX_LENGTH);
        List<URL> listOfFeedUrl = fitCodeLinesIntoUrls(splitedLines, CODESEARCH_QUERY_MAX_LENGTH);
        // call every URL and parse results in one map
        for (URL url : listOfFeedUrl) {
            CodeSearchFeed myFeed = codesearchService.getFeed(url, CodeSearchFeed.class);
            List<SearchResult> srList = createSearchResutl(myFeed, possibleCopyAndPastedCode);
            boolean found = (srList.size() > 0) && (postprocessor.isMatchFound(srList, possibleCopyAndPastedCode));

            toReturn = toReturn || found;

            if (found) {
                searchResults.addAll(srList);
            }
        }

        return toReturn;
    }

    /**
     * This method combine lines and produce URLs that will fit maxLength but
     * which are not greater then provided maxLength. For GoogleCodeSearch
     * length is 1024.
     * 
     * @param lines list of lines which will be combined to produce URL which
     *        length fit provided maxLength
     * @param maxLength maximum length of query which code search engine can
     *        manage
     * @return list of URL-s not longer then maxLength
     * @throws IOException
     */
    List<URL> fitCodeLinesIntoUrls(List<String> lines, int maxLength) throws IOException {
        List<URL> toRet = new ArrayList<URL>();
        URL current = null;
        URL temp = null;
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < lines.size(); i++) {
            sb.append(lines.get(i));
            // carriage return must be inserted between each two lines
            if (i != lines.size() - 1) {
                sb.append("\n");
            }
            temp = createUrl(sb.toString());

            if (temp.toString().length() <= maxLength) {
                current = temp;
            } else {
                toRet.add(current);
                sb.setLength(0);
                sb.append(lines.get(i));
                // carriage return must be inserted between each two lines
                if (i != lines.size() - 1) {
                    sb.append("\n");
                }
            }
        }
        toRet.add(createUrl(sb.toString()));
        return toRet;
    }

    /**
     * Append tokens from start position until end, inclusive.
     * 
     * @param tokens list of tokens to combine
     * @param start index of start position
     * @param end index of end position
     * @return appended tokens
     */
    private StringBuffer combineLines(List<String> tokens, int start, int end) {

        StringBuffer sb = new StringBuffer();
        for (int k = start; k < end; k++) {
            sb.append(tokens.get(k));
            sb.append("\n");
        }
        sb.append(tokens.get(end));
        return sb;
    }

    /**
     * for debugging
     * 
     * @param myFeed GoogleCodeSearch feed
     * @param entry Google CodeSearchEntry
     * @param out print stream for printing current information
     */
    private void printAdditionalInformation(CodeSearchFeed myFeed, CodeSearchEntry entry) {
        out.println("\tgetEtag: " + entry.getEtag());
        out.println("\tgetExtensionLocalName: " + entry.getExtensionLocalName());
        out.println("\tgetId: " + entry.getId());
        out.println("\tgetVersionId: " + entry.getVersionId());
        out.println("\tgetCategories: " + entry.getCategories().size());
        out.println("\tgetXmlBlob: " + entry.getXmlBlob());
        out.println("\tgetVersionId: " + entry.getVersionId());

        out.println("\tgetExtensionLocalName: " + entry.getFile().getExtensionLocalName());
        out.println("\tgetPackage: ");
        out.println("\t\t Name:" + entry.getPackage().getName());
        out.println("\tAuthor: " + entry.getAuthors().get(0).getName());
        out.println("\tTitle: " + entry.getTitle());
        out.println("Id: " + entry.getId());
        out.println("Link: " + entry.getHtmlLink().getHref());
        if (entry.getRights() != null)
            out.println("\tLicense:" + entry.getRights().getPlainText());
        for (Match m : entry.getMatches()) {
            out.println("\t\tMatch " + m.getLineNumber() + ": " + m.getLineText().getPlainText());
        }
    }

    /**
     * Parse GoogleCodeSearch feed and pack retrieved information in proper to
     * us form.
     * 
     * @param myFeed GoogleCodeSearch feed
     * @param possibleCopyAndPastedCode code to be checked
     * @return list of search result retrieved from GoogleCodeSearch feed
     */
    private List<SearchResult> createSearchResutl(CodeSearchFeed myFeed, String possibleCopyAndPastedCode) {
        List<SearchResult> toRet = new ArrayList<SearchResult>();
        for (CodeSearchEntry entry : myFeed.getEntries()) {
            SearchResult searchResult = new SearchResult();
            searchResult.setEngine("GoogleCodeSearch");
            searchResult.setCodeForQuery(possibleCopyAndPastedCode);
            searchResult.setLink(entry.getHtmlLink().getHref());
            searchResult.setProjectName(entry.getPackage().getName());
            searchResult.setOwner(entry.getAuthors().get(0).getName());
            searchResult.setClassName(entry.getTitle().getPlainText().replaceFirst(".*/", ""));
            searchResult.setPackageName(entry.getTitle().getPlainText().replace(searchResult.getClassName(), ""));
            searchResult.setCodeInClass(entry.getTextContent().getContent().getPlainText());
            if (entry.getRights() != null)
                searchResult.setLicence(entry.getRights().getPlainText());
            for (Match m : entry.getMatches()) {
                searchResult.getMatchedCode().add(m.getLineText().getPlainText());
            }
            toRet.add(searchResult);

            printAdditionalInformation(myFeed, entry);
        }

        return toRet;
    }

    // this is bug-fix of bug
    // http://code.google.com/p/apache-rat-pd/source/detail?r=51#
    /**
     * Generate list of lines that produce URL which length is smaller then
     * provided maxLength.
     * 
     * @param lines list of strings to filter
     * @param maxLength maximum length of URL produced from one string
     * @return list of lines that produce URL which length is smaller then
     *         provided maxLength
     */
    private List<String> generateLinesWithProperLength(String[] lines, int maxLength) {
        List<String> toRet = new ArrayList<String>();
        for (String line : lines) {
            try {
                if (createUrl(line).toString().length() <= maxLength) {
                    toRet.add(line);
                } else {
                    // TODO add better way to split lines with length of URL
                    // produced from them longer then maxLength. Something like
                    // splitting
                    // on dot sign or blank. Splitting in middle of the word is
                    // not best solution but it is universal. It must exist but
                    // only if all other solution are not successful.
                    toRet.addAll(splitOversizedLines(line, maxLength));
                }
            } catch (Exception e) {
                // we do not want to stop execution because of one line
                // anyway, this will be newer happened
                System.err.println("Error creating URL from string " + line + ", continuing...");
            }
        }
        return toRet;
    }

    /**
     * Splits line into multiple lines which satisfy expected maximum URL length
     * 
     * @param line line to split if URL created from it is too long
     * @param maximumLength maximum length of URL created from this line
     * @return list of lines which fit expected max URL length
     * @throws MalformedURLException
     */
    public List<String> splitOversizedLines(String line, int maximumLength) throws MalformedURLException {

        ArrayList<String> toRet = new ArrayList<String>();

        StringBuffer sb = new StringBuffer();
        String current = "";
        for (int i = 0; i < line.length(); i++) {

            sb.append(line.charAt(i));

            if (createUrl(sb.toString()).toString().length() <= maximumLength) {
                current = sb.toString();
            } else {
                toRet.add(current);
                sb.setLength(0);
                sb.append(line.charAt(i));
            }
        }
        toRet.add(sb.toString());
        return toRet;
    }

}
