/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.engines.google;

/**
 * This class is much better in matching code then RegexGenerator. It can very
 * correctly match same lines. This multi line matching is characteristic for
 * Google Code search. But it is sensitive to new lines - if line have break,
 * this class cannot match it.
 * 
 * @author maka
 */
public class MultilineRegexGenerator extends RegexGenerator {

    private static final String LINE_BEGINNING_REGEX = "^";
    private static final String LINE_END_REGEX = "$";
    private static final String BLANK_MARK_FOR_URL = " ";

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.apache.rat.pd.engines.google.RegexGenerator#stringToRegex(java.lang
     * .String)
     */
    @Override
    public String stringToRegex(String sourceCode) {
        String toRet = "";
        String[] lines = sourceCode.split("\n *");
        lines = excludeBlankLines(lines);
        for (int i = 0; i < lines.length; i++) {
            toRet += LINE_BEGINNING_REGEX + ZERO_OR_MORE_BLANK_SPACE + super.stringToRegex(lines[i]);
            // for current line, it is better not to add LINE_END_REGEX.
            // we don't know if end of real line in code is reached
            if (i != lines.length - 1) {
                toRet += ZERO_OR_MORE_BLANK_SPACE + LINE_END_REGEX + BLANK_MARK_FOR_URL;
            }
        }
        return toRet;
    }

    /**
     * Exclude blank lines from array.
     * 
     * @param lines list of lines of code
     * @return array of code lines without empty ones
     */
    private String[] excludeBlankLines(String[] lines) {
        int lentgh = lines.length;
        // determining new array length
        for (String string : lines) {
            if (string.trim().isEmpty()) {
                lentgh--;
            }
        }
        // copy non-blank lines
        String[] toRet = new String[lentgh];
        int index = 0;
        for (String line : lines) {
            if (!line.trim().isEmpty()) {
                toRet[index] = line;
                index++;
            }
        }
        return toRet;
    }
}
