/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.engines.google;

/**
 * It make regular expressions in POSIX extended regular expression syntax
 * standard. This class can generate regular expressions understandable to
 * GoogleCodeSearch parsing provided source code. It replace reserved symbols in
 * POSIX extended regular expression syntax with proper regular expressions, and
 * then replace regex matching whitespace to ensure that our search is non
 * sensitive to whitespace and new line characters.
 * 
 * @author maka
 */
public class RegexGenerator {
    // special signs in regex
    protected final static String L_PARENTHESIS = "\\(";
    protected final static String R_PARENTHESIS = "\\)";
    protected final static String L_SQUARE_BRACKET = "\\[";
    protected final static String R_SQUARE_BRACKET = "\\]";
    protected final static String L_CURLY_BRACKET = "\\{";
    protected final static String R_CURLY_BRACKET = "\\}";
    protected final static String DOT = "\\.";
    protected final static String QUESTION_MARK = "\\?";
    protected final static String ASTERIX = "\\*";
    protected final static String POWER = "\\^";
    protected final static String DOLLAR_SIGN = "\\$";
    protected final static String PLUS_SIGN = "\\+";
    protected final static String MINUS_SIGN = "\\-";
    protected final static String VERTICAL_LINE = "\\|";
    protected final static String BACK_SLASH = "\\";
    // non literals
    protected final static String LES_THEN = "<";
    protected final static String GREAT_THEN = ">";
    protected final static String SEMICOLOMN = ";";
    protected final static String COLON = ":";
    protected final static String EQUALS = "=";
    protected final static String SLASH = "/";
    protected final static String DOUBLE_QUOTES = "\"";

    // regex for matching
    protected final static String ZERO_OR_MORE_BLANK_SPACE = "(\\s?)+";
    protected final static String ONE_OR_MORE_BLANK_SPACE = "\\s+";
    // protected final static String NOT_A_CHARACTER = "";

    // other
    protected final static String BLANK_SPACE = " ";

    /**
     * this regex is better then '\s* when it is used on GoogleCodeSearch
     */
    protected final static String ZERO_OR_MORE_BLANK_SPACE_REGEX = "(\\\\s?)+";

    protected final static String[] regExpresions = { BACK_SLASH, L_PARENTHESIS, DOLLAR_SIGN, R_PARENTHESIS,
            L_SQUARE_BRACKET, R_SQUARE_BRACKET, L_CURLY_BRACKET, R_CURLY_BRACKET, DOT, QUESTION_MARK, ASTERIX, POWER,
            PLUS_SIGN, MINUS_SIGN, VERTICAL_LINE, LES_THEN, GREAT_THEN, SEMICOLOMN, COLON, EQUALS, SLASH, DOUBLE_QUOTES };

    /**
     * Returns regular expression of source code
     * 
     * @param sourceCode code from which regex are generated
     * @return regex generated from source code
     */
    public String stringToRegex(String sourceCode) {
        // add whitespace before and after any non-character
        sourceCode = addWhitespaceAroundNonLiteral(sourceCode);
        // replace non literal with proper regex
        sourceCode = replaceNonLiteralWithProperRegex(sourceCode);
        // replace blank spaces with proper regex
        sourceCode = replaceWhitespaceWithRegex(sourceCode);
        return sourceCode;
    }

    /**
     * Replace strings matching whitespace with proper regex
     * 
     * @param code string with multiple blank spaces
     * @return string without multiple blank spaces
     */
    protected String replaceWhitespaceWithRegex(String code) {
        code = code.replaceAll(" ", ZERO_OR_MORE_BLANK_SPACE_REGEX);
        return code;
    }

    /**
     * Replace reserved symbols in POSIX extended regular expression syntax with
     * proper regular expressions
     * 
     * @param code string with symbols listed in RegexGenerator.regExpresions
     *        array
     * @return string with regular expressions
     */
    protected String replaceNonLiteralWithProperRegex(String code) {
        for (String regex : regExpresions) {
            if (regex.equals(DOLLAR_SIGN)) {
                // for some reason, String.replaceAll("\$", "anything") always
                // breaks
                code = code.replace("$", "\\$");
            } else if (regex.equals(BACK_SLASH)) {
                // bug-fix for code consist backslash
                code = code.replace(BACK_SLASH, "\\\\");
            } else {
                code = code.replaceAll(regex, BACK_SLASH + regex);
            }
        }
        return code;
    }

    /**
     * Add whitespace around non-literal
     * <p>
     * for example: "main(String[] args)" will be "main ( String [ ] args )"
     * 
     * @param code string
     * @return string with whitespace added around non-literal
     */
    protected String addWhitespaceAroundNonLiteral(String code) {
        for (String regex : regExpresions) {
            // in words beginning with
            // "\" it should not be whitespace inserted after "\".
            if (!regex.equals(BACK_SLASH)) {
                code = code.replaceAll(regex, BLANK_SPACE + regex + BLANK_SPACE);
            }
        }
        code = code.replaceAll(ONE_OR_MORE_BLANK_SPACE, BLANK_SPACE);
        return code.trim();
    }
}
