/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.comment;

import java.io.PrintStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.rat.pd.heuristic.HeuristicCheckerResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;

/**
 * This is base class for all comment heuristic checkers.
 * 
 * @author maka
 */
public class CommentHeuristicChecker implements IHeuristicChecker {

    protected final String COMMENT_REGEX;
    private final int limit;
    private final String language;
    private final PrintStream out;

    /**
     * @param regex regular expression to match comment in current language
     * @param limit minimal length of comment which will be considered
     * @param language programming language
     * @param out print stream for logging purposes
     */
    public CommentHeuristicChecker(String regex, int limit, String language, PrintStream out) {
        this.COMMENT_REGEX = regex;
        this.limit = limit;
        this.language = language;
        this.out = out;
    }

    /**
     * Method will return false if there are some characters before comment or
     * after comment. Depends on limit property, function can return false if
     * matched comment length is smaller then limit value.
     * 
     * @param codeToBeChecked
     * @return true if this is comment.
     */
    @Override
    public HeuristicCheckerResult checkByHeuristic(String codeToBeChecked) {
        HeuristicCheckerResult toret = new HeuristicCheckerResult(false, true, codeToBeChecked, this.getClass());

        // Create a pattern to match comments
        Pattern p = Pattern.compile(COMMENT_REGEX, Pattern.MULTILINE);
        // Run matches
        Matcher m = p.matcher(codeToBeChecked);
        // there must be no characters after comment
        if (m.find()) {
            String found = m.group();
            if (codeToBeChecked.endsWith(found)) {
                out.println("Found comment: " + found);
                boolean isSearchNeaded = found.length() > limit ? true : false;
                toret = new HeuristicCheckerResult(isSearchNeaded, false, found, this.getClass());
            }
        }
        return toret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.rat.pd.heuristic.IHeuristicChecker#getLanguage()
     */
    @Override
    public String getLanguage() {
        return this.language;
    }
}
