/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.functions;

import java.io.PrintStream;

/**
 * Function in C consists of function definition and function body. There are
 * special constructions called "inline functions" and "macro functions" too.
 * More info on:
 * <p>
 * <li>{@link} http://www2.its.strath.ac.uk/courses/c/
 * <li>{@link}http://www.greenend.org.uk/rjk/2003/03/inline.html
 * 
 * @author maka
 */
public class CFunctionHeuristicChecker extends FunctionHeuristicChecker {

    private static final String OR_REGEX = "|";
    private static final String C_OPENED_BRACKET = "\\{";
    private static final String C_CLOSED_BRACKET = "\\}";

    /**
     * Function in C consists of function definition and function body. More
     * info on:
     * <p>
     * <li>{@link} http://www2.its.strath.ac.uk/courses/c/ <li>{@link}
     * http://www.greenend.org.uk/rjk/2003/03/inline.html
     */
    private static final String C_FUNCTION_REGEX = "\\w+\\s+[*&]?\\s*\\w+ *\\([^()]*?\\)(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match macro in C which are multilines. For
     * example: #define SWAP(a, b) {
     * <p>
     * a ^= b;
     * <p>
     * b ^= a;
     * <p>
     * a ^= b; }
     */
    private static final String C_MACRO_REGEX_1 = "#define +\\w+ *\\([\\s\\S]*\\)(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match macro in C which are one-line. For example:
     * <p>
     * #define INCREMENT(x) x++
     */
    private static final String C_MACRO_REGEX_2 = "#define +\\w+ *\\([\\s\\S]*\\).*[\\n\\r]";

    /**
     * This regular expression match macro in C which are multiline but have no
     * parameters. For example:
     * <p>
     * #define UPDBITS {s->bitb=b;s->bitk=k;}
     */
    private static final String C_MACRO_REGEX_3 = "#define +\\w+(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match macros in C. More info on: {@link }
     * http://www.cprogramming.com/tutorial/cpreprocessor.html
     */
    private static final String C_MACRO_REGEX = "(" + C_MACRO_REGEX_1 + ")" + OR_REGEX + "(" + C_MACRO_REGEX_2 + ")"
            + OR_REGEX + "(" + C_MACRO_REGEX_3 + ")";

    /**
     * This regular expression match inline functions in C which are multiline
     * and have parametrs. For example:
     * <p>
     * inline void Recycle( char* aBuffer) { nsMemory::Free(aBuffer); }
     */

    private static final String C_INLINE_FUNCTION_1 = "inline(\\s)*\\w+(\\s*)(.*)\\([^()]*?\\)(\\s)*\\{[\\s\\S]*\\}[\n\r]*";
    /**
     * This regular expression match inline functions in C which have no
     * parametrs.
     */
    private static final String C_INLINE_FUNCTION_2 = "inline(\\s)*\\w+(\\s*)(.*)\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match inline functions in C. More info on :
     * {@link} http://www.cprogramming.com/tutorial/lesson13.html
     */
    private static final String C_INLINE_FUNCTION = "(" + C_INLINE_FUNCTION_1 + ")" + OR_REGEX + "("
            + C_INLINE_FUNCTION_2 + ")";

    /**
     * This regular expression match functions,inline functions and macros in C,
     */
    private static final String FUNCTION_REGEX = "(" + C_FUNCTION_REGEX + ")" + OR_REGEX + C_MACRO_REGEX + OR_REGEX
            + C_INLINE_FUNCTION;

    /**
     * @param limit minimal length of function which will be considered
     * @param out print stream for logging purposes
     */
    public CFunctionHeuristicChecker(int limit, PrintStream out) {
        super(limit, FUNCTION_REGEX, C_OPENED_BRACKET, C_CLOSED_BRACKET, "c", out);
    }

}
