/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.functions;

import java.io.PrintStream;

/**
 * Function in C++ consists of function definition and function body. There are
 * special constructions called "inline functions" and "macro functions"too.
 * 
 *@author maka
 */
public class CPPFunctionHeuristicChecker extends FunctionHeuristicChecker {

    private static final String OR_REGEX = "|";
    private static final String CPP_CLOSED_BRACKET = "\\}";
    private static final String CPP_OPENED_BRACKET = "\\{";

    /**
     * This regular expression match functions in C++.
     */
    private static final String CPP_FUNCTION_REGEX_1 = "\\w+\\s+[*&]?\\s*\\w+ *\\([^()]*?\\)(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match functions in C++ which have mark "::". For
     * example:
     * <p>
     * MyType MyType::Add(const MyType & rhs) { return MyType(itsVal+
     * rhs.GetItsVal()); }
     */
    private static final String CPP_FUNCTION_REGEX_2 = "^[\t ]*[\\w+ \\*&]+\\s*\\:\\:\\s*[*&~]?\\s*\\w+ *\\([^()]*?\\)( *const *)?(\\s)*\\{[\\s\\S]*\\}[\n\r]*";
    private static final String CPP_FUNCTION_REGEX = "(" + CPP_FUNCTION_REGEX_1 + ")|(" + CPP_FUNCTION_REGEX_2 + ")";

    /**
     * This regular expression match macros in C++ which are multilines. For
     * example: #define SWAP(a, b) {
     * <p>
     * a ^= b;
     * <p>
     * b ^= a;
     * <p>
     * a ^= b; }
     */
    private static final String CPP_MACRO_REGEX_1 = "#define +\\w+ *\\([\\s\\S]*\\)\\s*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match macro in C++ which are one-line. For
     * example:
     * <p>
     * #define INCREMENT(x) x++
     */
    private static final String CPP_MACRO_REGEX_2 = "#define +\\w+ *\\([\\s\\S]*\\)*.*[\\n\\r]";

    /**
     * This regular expression match macros in C++.
     */
    private static final String CPP_MACRO_REGEX = "(" + CPP_MACRO_REGEX_1 + ")" + OR_REGEX + "(" + CPP_MACRO_REGEX_2
            + ")";

    /**
     * This regular expression match inline functions in C++ which are multiline
     * and have parametrs. For example: inline void Recycle( char* aBuffer) {
     * nsMemory::Free(aBuffer); }
     */
    private static final String CPP_INLINE_FUNCTION_1 = "inline\\s+\\w+\\s+(.*)\\([^()]*\\)(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match inline functions in C++ which have no
     * parametrs.
     */
    private static final String CPP_INLINE_FUNCTION_2 = "inline +\\w+(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression matches inline functions in C++ which contains
     * mark "::". For example:inline size_t SearchResultMessageArg::nTried() {
     * return filename_.size(); }
     */
    private static final String CPP_INLINE_FUNCTION_3 = "inline\\s+\\w+\\s+\\w+\\:\\:\\w+\\s*\\([^()]*\\)(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression matches inline functions in C++ which contains
     * mark "::" and,after parameters, key word const. For example: inline
     * size_t SearchResultMessageArg::nTried() const{ return filename_.size();}
     */
    private static final String CPP_INLINE_FUNCTION_4 = "inline\\s+\\w+\\s+\\w+\\:\\:\\w+\\s*\\([^()]*\\) *const(\\s)*\\{[\\s\\S]*\\}[\n\r]*";

    /**
     * This regular expression match inline functions in C++.
     */
    private static final String CPP_INLINE_FUNCTION = "(" + CPP_INLINE_FUNCTION_1 + ")" + OR_REGEX + "("
            + CPP_INLINE_FUNCTION_2 + ")" + OR_REGEX + "(" + CPP_INLINE_FUNCTION_3 + ")" + OR_REGEX + "("
            + CPP_INLINE_FUNCTION_4 + ")";

    /**
     * This regular expression match functions,inline functions and macros in
     * C++,
     */
    private static final String FUNCTION_REGEX = CPP_FUNCTION_REGEX + OR_REGEX + CPP_MACRO_REGEX + OR_REGEX
            + CPP_INLINE_FUNCTION;

    /**
     * @param limit minimal length of function which will be considered
     * @param out print stream for logging purposes
     */
    public CPPFunctionHeuristicChecker(int limit, PrintStream out) {
        super(limit, FUNCTION_REGEX, CPP_OPENED_BRACKET, CPP_CLOSED_BRACKET, "c++", out);
    }

}
