/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.functions;

import java.io.PrintStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.rat.pd.heuristic.HeuristicCheckerResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;

/**
 * There are two types of subprograms in Fortran: functions and subroutines.
 * Functions are started with a line that includes the type of value the
 * function will return,word FUNCTIONS the function name, and the list of
 * arguments the function takes as inputs. Subroutines begin with a line that
 * includes the word SUBROUTINE, the name of the subroutine, and the arguments
 * for the subroutine. More info on:
 * http://www.chem.ox.ac.uk/fortran/subprograms.html
 * 
 * @author maka
 */
public class FortranFunctionHeuristicChecker implements IHeuristicChecker {
    /**
     * This regular expression matches key word FUNCTION which is case
     * insensitive followed by one or more blanks.
     */
    public static final String FUNCTION_KEYWORD = "(?i:FUNCTION) +";

    /**
     * This regular expression matches key word SUBROUTINE which is case
     * insensitive followed by one or more blanks.
     */
    public static final String SUBROUTINE_KEYWORD = "(?i:SUBROUTINE) +";

    /**
     * This regular expression matches name of a functions starting with a key
     * word FUNCTION
     */
    public static final String FUNCTION_NAME_REGEX = "(?i:FUNCTION) +\\w+";

    /**
     * This regular expression matches name of a subroutine starting with a key
     * word SUBROUTINE.
     */
    public static final String SUBROUTINE_NAME_REGEX = "(?i:SUBROUTINE) +\\w+";

    /**
     * This regular expression matches functions header in Fortran.
     */
    public static final String FUNCTION_HEADER_REGEX = "\\w+ +(?i:FUNCTION) +\\w+ *\\(.*\\).*\\n";

    /**
     * This regular expression matches subroutines header in Fortran.
     */
    public static final String SUBROUTINE_HEADER_REGEX = " *(?i:SUBROUTINE) +\\w+ *\\(.*\\).*\\n";

    private int limit;
    private final String language;
    private final PrintStream out;

    public FortranFunctionHeuristicChecker(int limit, PrintStream out) {

        this.limit = limit;
        this.language = "fortran";
        this.out = out;
    }

    @Override
    public HeuristicCheckerResult checkByHeuristic(String codeToBeChecked) {
        HeuristicCheckerResult toret = new HeuristicCheckerResult(false, true, codeToBeChecked, this.getClass());
        boolean isSearchNeaded = false;
        String functionHeader = getFunctionHeader(codeToBeChecked, FUNCTION_HEADER_REGEX);
        String functionName;
        if (functionHeader != null) {
            functionName = getFunctionName(functionHeader, FUNCTION_KEYWORD, FUNCTION_NAME_REGEX);
            // TODO This regular expression can hung up an application.
            String functionEndRegex = // "(?i:END) +(?i:FUNCTION) +" +
            "" + functionName;
            isSearchNeaded = isFortranFunction(codeToBeChecked, FUNCTION_HEADER_REGEX, functionEndRegex);
        } else {

            functionHeader = getFunctionHeader(codeToBeChecked, SUBROUTINE_HEADER_REGEX);
            if (functionHeader != null) {
                functionName = getFunctionName(functionHeader, SUBROUTINE_KEYWORD, SUBROUTINE_NAME_REGEX);

                isSearchNeaded = isFortranFunction(codeToBeChecked, SUBROUTINE_HEADER_REGEX, functionName);
            }
        }
        toret = new HeuristicCheckerResult(isSearchNeaded, false, codeToBeChecked, this.getClass());
        return toret;
    }

    /**
     * Method will return false if there are some characters before subprograms
     * or after subprogram. Depends on limit property, function can return false
     * if matched subprogram length is smaller then limit value.
     * 
     * @param codeToBeChecked code currently being checked
     * @param functionHeader regular expression which match header of a
     *        subprogram in Fortran
     * @param functionEndRegex regular expression which match end of a
     *        subprogram in Fortran
     * @return true if this is Fortran subprogram.
     */
    boolean isFortranFunction(String codeToBeChecked, String functionHeader, String functionEndRegex) {
        boolean toret = false;
        String functionRegex = functionHeader + "[\\s\\S]*" + functionEndRegex;
        out.println(functionRegex);
        Pattern p = Pattern.compile(functionRegex, Pattern.MULTILINE);
        // Run matches
        Matcher m = p.matcher(codeToBeChecked);

        if (m.find()) {
            String found = m.group();
            if (codeToBeChecked.equals(found)) {
                out.println("Found function: " + found);
                toret = found.length() > limit ? true : false;
            }
        }

        return toret;
    }

    /**
     * Method will return functions name from a functions header
     * 
     * @param functionHeader regular expression which match header of a
     *        subprogram in Fortran
     * @param keywordRegex keyword for function and subroutine
     * @param functionNameRegex regular expression which match keyword and name
     *        of a subprogram in Fortran
     * @return functions name
     */
    String getFunctionName(String functionHeader, String keywordRegex, String functionNameRegex) {
        String toret = null;

        Pattern p = Pattern.compile(functionNameRegex, Pattern.MULTILINE & Pattern.CASE_INSENSITIVE);
        // Run matches
        Matcher m = p.matcher(functionHeader);

        if (m.find()) {
            toret = m.group();
            toret = toret.replaceFirst(keywordRegex, "");
        }

        return toret;
    }

    /**
     * Method will return functions header from a string. Functions header is
     * matched by a regular expression regex.
     * 
     * @param codeToBeChecked code currently being checked
     * @param regex regular expression which match subprogram's header
     * @return functions header if it functions
     */
    String getFunctionHeader(String codeToBeChecked, String regex) {
        String toret = null;

        Pattern p = Pattern.compile(regex, Pattern.MULTILINE & Pattern.CASE_INSENSITIVE);
        // Run matches
        Matcher m = p.matcher(codeToBeChecked);

        if (m.find()) {
            toret = m.group();
        }

        return toret;
    }

    @Override
    public String getLanguage() {
        return this.language;
    }

}
