/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.functions;

import java.io.PrintStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.rat.pd.heuristic.HeuristicCheckerResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;

/**
 * This is basic class for all programming languages which have C-like
 * functions.
 * 
 * @author maka
 */
public class FunctionHeuristicChecker implements IHeuristicChecker {
    private final String CLOSED_BRACKET;
    private final String OPENED_BRACKET;
    private final String FUNCTION_REGEX;
    private final int limit;
    private final String language;
    private final PrintStream out;

    /**
     * @param limit minimal length of comment which will be considered
     * @param regex regular expression to match comment in current language
     * @param openedBrackets opened brackets in particular programming language(
     *        for example "{" in c)
     * @param closedBrackets closed brackets in particular programming language(
     *        for example "}" in c)
     * @param language programming language
     * @param out print stream for logging purposes
     */
    public FunctionHeuristicChecker(int limit, String regex, String openedBrackets, String closedBrackets,
            String language, PrintStream out) {
        this.limit = limit;
        this.FUNCTION_REGEX = regex;
        this.CLOSED_BRACKET = closedBrackets;
        this.OPENED_BRACKET = openedBrackets;
        this.language = language;
        this.out = out;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.apache.rat.pd.heuristic.IHeuristicChecker#checkByHeuristic(java.lang
     * .String)
     */
    public HeuristicCheckerResult checkByHeuristic(String codeToBeChecked) {
        HeuristicCheckerResult toret = new HeuristicCheckerResult(false, true, codeToBeChecked, this.getClass());

        Pattern p = Pattern.compile(FUNCTION_REGEX, Pattern.MULTILINE);
        // Run matches
        Matcher m = p.matcher(codeToBeChecked);

        if (m.find()) {
            String found = m.group();
            if (codeToBeChecked.endsWith(found)) {
                boolean isSearchNeaded = found.length() > limit && filterWrongMatches(found) ? true : false;
                toret = new HeuristicCheckerResult(isSearchNeaded, false, found, this.getClass());
                out.println("Found function: " + found);
            }
        }
        return toret;
    }

    /**
     * This method can filter correct functions from incomplete ones. Function
     * must have equal number of opened and closed segment symbols. Java
     * example: public String toString(){ return super.toString(); } have one
     * opened and one closed brackets. This function have correct syntax.
     * 
     * @param codeToBeChecked
     * @return true if codeToBeChecked is correct function
     */
    private boolean filterWrongMatches(String codeToBeChecked) {
        int numberOfOpenedBrackets = count(OPENED_BRACKET, codeToBeChecked);
        int numberOfClosedBrackets = count(CLOSED_BRACKET, codeToBeChecked);
        return numberOfOpenedBrackets == numberOfClosedBrackets;
    }

    // TODO think about brackets in comments inside functions
    private int count(String regex, String codeToBeChecked) {
        int toret = 0;
        Pattern p = Pattern.compile(regex, Pattern.MULTILINE);
        Matcher m = p.matcher(codeToBeChecked);
        while (m.find()) {
            toret++;
        }
        return toret;
    }

    @Override
    public String getLanguage() {
        return this.language;
    }

}
