/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.misspellings;

import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.Validate;
import org.apache.rat.pd.heuristic.HeuristicCheckerResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;

/**
 * This class can examine source code on misspellings.
 * 
 * @author maka
 */
public class MisspellingsHeuristicChecker implements IHeuristicChecker {

    private IDictionary spellChecker;
    private final String language;
    private final PrintStream out;

    /**
     * @param dictionary dictionary
     * @param language programming language
     * @param out print stream for logging purposes
     */
    public MisspellingsHeuristicChecker(IDictionary dictionary, String language, PrintStream out) {
        this.spellChecker = dictionary;
        this.language = language;
        this.out = out;
    }

    /**
     * This method will return true if any word in source code is misspelled.
     * Composed words are first decomposed by: <li>CamelCase, so myFunction ->
     * [my, Function] <li>underscore character, so my_Function -> [my, Function]
     * (non-Javadoc)
     * 
     * @see org.apache.rat.pd.heuristic.IHeuristicChecker#checkByHeuristic(java.lang.String)
     */
    @Override
    public HeuristicCheckerResult checkByHeuristic(String codeToBeChecked) {
        HeuristicCheckerResult toRet = new HeuristicCheckerResult(false, true, codeToBeChecked, this.getClass());
        boolean isCheckingOnInternetNeaded = true;
        List<String> words = filterWordsFromCode(codeToBeChecked);
        for (String word : words) {
            try {
                isCheckingOnInternetNeaded = isCheckingOnInternetNeaded && isWordCorrect(decomposeCombinedWord(word));
            } catch (Exception e) {
                isCheckingOnInternetNeaded = false;
                out.println("Error in decomposing word:" + word + " We consider that it is misspelled.");
            }
        }
        toRet = new HeuristicCheckerResult(!isCheckingOnInternetNeaded, false, codeToBeChecked, this.getClass());
        return toRet;
    }

    /**
     * This method can extract words from code. Words are strings without digits
     * inside, but underscore characters are not excluded.
     * 
     * @param codeToBeChecked
     * @return list of words in codeToBeChecked
     */
    List<String> filterWordsFromCode(String codeToBeChecked) {
        List<String> toRet = new ArrayList<String>();
        Pattern p = Pattern.compile("[a-zA-Z_]+", Pattern.MULTILINE);
        Matcher m = p.matcher(codeToBeChecked);
        while (m.find()) {
            toRet.add(m.group());
        }
        return toRet;
    }

    boolean isWordCorrect(List<String> combinedWords) throws IOException {
        boolean toRet = true;
        for (String word : combinedWords) {
            // it is assumed that words with capital case are equal to word with
            // all small case in dictionary
            toRet = toRet && spellChecker.exist(word.toLowerCase());
            out.println(word + " " + toRet);
        }
        return toRet;
    }

    /**
     * Combined word can be combined with usage of CamelCase or underscore
     * character. This method will return list of simple words from such
     * combined word.
     * 
     * @param combinedWord
     * @return list of decomposed words
     * @throws IllegalArgumentException
     */
    List<String> decomposeCombinedWord(String combinedWord) throws IllegalArgumentException {
        Validate.isTrue(combinedWord.matches("[a-zA-Z_]+"));
        List<String> toRet = new ArrayList<String>();
        toRet.add(combinedWord);
        toRet = decomposeUnderscoreCombinedWords(toRet);
        toRet = decomposeCamelCombinedWords(toRet);
        return toRet;
    }

    /**
     * Combined word can be combined with usage of underscore character. This
     * method will return list of simple words from such combined word.
     * 
     * @param combinedWord
     * @return list of decomposed words
     */
    List<String> decomposeUnderscoreCombinedWords(List<String> combinedWord) {
        List<String> toRet = new ArrayList<String>();
        for (String word : combinedWord) {
            toRet.addAll(new ArrayList<String>(Arrays.asList(word.split("_+"))));
            // sometimes empty string is entered in list
            // this remove such entries
            toRet.remove("");
        }

        return toRet;
    }

    /**
     * This function will decompose camel combined words in component words.
     * Example: "bigInteger" will be "big integer"
     * 
     * @param combinedWord
     * @return list of decomposed words
     */
    List<String> decomposeCamelCombinedWords(List<String> combinedWord) {
        List<String> toRet = new ArrayList<String>();
        for (String word : combinedWord) {
            // extract only first part with small caps of string if it exists
            // example: thisIsMyName -> this
            Pattern p = Pattern.compile("\\b[a-z]+");
            Matcher m = p.matcher(word);
            if (m.find()) {
                toRet.add(m.group());
            }

            // extract all parts of string which starts with capital case to
            // other capital case
            // example: IsMyName-> Is,My,Name
            p = Pattern.compile("[A-Z][a-z]*");
            m = p.matcher(word);
            while (m.find()) {
                toRet.add(m.group());
            }
        }
        return toRet;
    }

    @Override
    public String getLanguage() {
        return this.language;
    }
}
