/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.report;

import java.util.List;
import java.util.regex.Matcher;

import org.apache.rat.pd.engines.SearchResult;
import org.apache.rat.pd.engines.google.RegexGenerator;
import org.apache.rat.pd.util.FileManipulator;

/**
 * This class can generate html report.
 * 
 * @author maka
 */
public class HtmlReportGenerator extends Report {
    private String reportEntryTemplate;
    private String reportTemplate;
    private String searchResultTemplate;

    private final String REPORT_LIST = "REPORT_LIST";
    private final String SEARCH_RESULT_NAME = "SEARCH_RESULT_NAME";
    private final String SEARCH_RESULT_CLASS_NAME = "SEARCH_RESULT_CLASS_NAME";
    private final String SEARCH_RESULT_PACKAGE_NAME = "SEARCH_RESULT_PACKAGE_NAME";

    private final String SEARCH_RESULT_CODE = "SEARCH_RESULT_CODE";
    private final String SEARCH_RESULT_LICENSE = "SEARCH_RESULT_LICENSE";
    private final String SEARCH_RESULT_OWNER = "SEARCH_RESULT_OWNER";
    private final String SEARCH_RESULT_LANGUAGE = "SEARCH_RESULT_LANGUAGE";
    private final String SEARCH_RESULT_LINK = "SEARCH_RESULT_LINK";
    private final String SEARCH_RESULT_ENGINE = "SEARCH_RESULT_ENGINE";
    private final String FILE_NAME = "FILE_NAME";
    private final String CODE_TO_BE_CHECKED = "CODE_TO_BE_CHECKED";
    private final String SEARCH_RESULTS_LIST = "SEARCH_RESULTS_LIST";

    private RegexGenerator regexGenerator = new RegexGenerator();

    /**
     * Can generate explanation about plagiarized code in html format.
     */
    public HtmlReportGenerator() {
        super();
        this.reportEntryTemplate = FileManipulator.convertStreamToString(ClassLoader.getSystemClassLoader()
                .getResourceAsStream("report/reportEntryTemplate.html"));
        this.reportTemplate = FileManipulator.convertStreamToString(ClassLoader.getSystemClassLoader()
                .getResourceAsStream("report/reportTemplate.html"));
        this.searchResultTemplate = FileManipulator.convertStreamToString(ClassLoader.getSystemClassLoader()
                .getResourceAsStream("report/searchResultTemplate.html"));
    }

    /**
     * @return generate explanation about plagiarized code in html format.
     */
    @Override
    public String getReport() {
        String toRet = this.reportTemplate.replace(this.REPORT_LIST, createReportList());
        return toRet;
    }

    /**
     * @return create report list using html template
     */
    private String createReportList() {
        String toRet = "";
        for (ReportEntry reportEntry : raportList) {
            String current = this.reportEntryTemplate.replace(this.FILE_NAME, reportEntry.getFileName());

            current = current.replace(this.CODE_TO_BE_CHECKED, reportEntry.getCode().replaceAll("\\n", "<BR>")
                    .replaceAll("\\t", "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;").replaceAll(" ", "&nbsp;"));

            current = current.replace(this.SEARCH_RESULTS_LIST, createSearchResultList(reportEntry));

            toRet += current;
        }
        return toRet;
    }

    /**
     * @param reportEntry this object will be shown as HTML
     * @return create search result list using HTML template
     */
    private String createSearchResultList(ReportEntry reportEntry) {
        String toRet = "";
        for (SearchResult searchResult : reportEntry.getSearchResult()) {
            String current = this.searchResultTemplate.replaceAll(this.SEARCH_RESULT_CLASS_NAME, searchResult
                    .getClassName());
            current = current.replace(this.SEARCH_RESULT_NAME, searchResult.getProjectName());
            current = current.replace(this.SEARCH_RESULT_PACKAGE_NAME, searchResult.getPackageName());
            current = current.replace(this.SEARCH_RESULT_LICENSE, searchResult.getLicence());
            // TODO result code must be placed here - nor queried code
            current = current.replace(this.SEARCH_RESULT_CODE, formatBetter(searchResult.getMatchedCode(), searchResult
                    .getCodeForQuery()));
            current = current.replace(this.SEARCH_RESULT_OWNER, searchResult.getOwner());
            current = current.replace(this.SEARCH_RESULT_LANGUAGE, searchResult.getLanguage());
            current = current.replace(this.SEARCH_RESULT_LINK, searchResult.getLink());
            current = current.replace(this.SEARCH_RESULT_ENGINE, searchResult.getEngine());
            toRet += current;
        }
        return toRet;
    }

    /**
     * @param matchedCode list of code parts which will be highlighted
     * @param codeInClass code to format
     * @return code formated using HTML tags
     */
    protected String formatBetter(List<String> matchedCode, String codeInClass) {
        String toRet = codeInClass;
        // basic code highlighting
        for (int i = 0; i < matchedCode.size(); i++) {
        	//Matcher.quoteReplacement is here because bug fix. More information can be found on:
        	//http://cephas.net/blog/2006/02/09/javalangillegalargumentexception-illegal-group-reference-replaceall-and-dollar-signs/
            toRet = toRet.replaceAll("(?i:(" + regexGenerator.stringToRegex(matchedCode.get(i)) + "))",
                    "<b><span style=\"color: #800000\">" + Matcher.quoteReplacement(matchedCode.get(i)) + "</span></b>");
        }
        // basic code formating
        toRet = toRet.replaceAll("\\n", "<BR>").replaceAll("\\t", " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; ");
        return toRet;
    }

}
