/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.core;

import junit.framework.TestCase;

public class PdCommandLineTest extends TestCase {

    public void testNoArgs() {

        PdCommandLine pdCommandLine = new PdCommandLine(new String[] {}, System.out);
        assertNull("I assert that code base property is null", pdCommandLine.getCodebase());
        assertEquals("I assert that the size of engines list is 0", 0, pdCommandLine.getEngines().size());
        assertEquals("I assert that the size of list of heuristics is 0", 0, pdCommandLine.getHeuristics().size());
        assertNull("I assert that the value of report is not set", pdCommandLine.getReport());
        assertFalse("I assert that the flag is set to false", pdCommandLine.isAllArgumentsCorrect());
        assertNull("I assert that the language is not set", pdCommandLine.getLanguage());
        assertEquals(0, pdCommandLine.getWaitingTime());
    }

    public void testCodebaseArg() {
        PdCommandLine pdCommandLine = new PdCommandLine(new String[] { "-cb", "c:/src" }, System.out);
        assertEquals("c:/src", pdCommandLine.getCodebase());
        pdCommandLine = new PdCommandLine(new String[] { "-codebase", "c:/src" }, System.out);
        assertEquals("c:/src", pdCommandLine.getCodebase());
        pdCommandLine = new PdCommandLine(new String[] { "-cb", "c:/src", "c:/src2" }, System.out);
        assertEquals("c:/src", pdCommandLine.getCodebase());

    }

    public void testAllArgs() {
        String[] args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-engines", "myEngine",
                "-heur", "myHeuristic", "-lang", "java", "-w", "2450" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that codebase is c:/src", "c:/src", pdCommandLine.getCodebase());
        assertEquals("rep.txt", pdCommandLine.getReport());
        assertEquals(23, pdCommandLine.getLimit());
        assertEquals("myEngine", pdCommandLine.getEngines().get(0));
        assertEquals("myHeuristic", pdCommandLine.getHeuristics().get(0));
        assertEquals("java", pdCommandLine.getLanguage());
        assertEquals(2450, pdCommandLine.getWaitingTime());

        args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine", "myEngine2",
                "-heur", "myHeuristic", "-language", "c++" };

        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there are two engines", 2, pdCommandLine.getEngines().size());
        assertEquals("I assert that first engine is myEngine", "myEngine", pdCommandLine.getEngines().get(0));
        assertEquals("I assert that second engine is myEngine2", "myEngine2", pdCommandLine.getEngines().get(1));
        assertEquals("c++", pdCommandLine.getLanguage());

        args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-engines", "myEngine,myEngine2",
                "-heur", "myHeuristic", "-language", "pascal" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("myEngine", pdCommandLine.getEngines().get(0));
        assertEquals("myEngine2", pdCommandLine.getEngines().get(1));
        assertEquals("pascal", pdCommandLine.getLanguage());

        args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-engines", "myEngine,myEngine2",
                "-heur", "myHeuristic", "-engines", "myEngine3" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there are 3 engines", 3, pdCommandLine.getEngines().size());
        assertEquals("I assert that first engine is myEngine", "myEngine", pdCommandLine.getEngines().get(0));
        assertEquals("I assert that second engine is myEngine2", "myEngine2", pdCommandLine.getEngines().get(1));
        assertEquals("I assert that third engine is myEngine3", "myEngine3", pdCommandLine.getEngines().get(2));
        args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-engines", "myEngine,myEngine2",
                "-heur", "myHeuristic", "-e", "myEngine3", "-heuristics", "myHeuristic2" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there are 3 engines", 3, pdCommandLine.getEngines().size());
        assertEquals("I assert that first engine is myEngine", "myEngine", pdCommandLine.getEngines().get(0));
        assertEquals("I assert that second engine is myEngine2", "myEngine2", pdCommandLine.getEngines().get(1));
        assertEquals("I assert that third engine is myEngine3", "myEngine3", pdCommandLine.getEngines().get(2));
        assertEquals("I assert that there are 2 heuristics", 2, pdCommandLine.getHeuristics().size());
        assertEquals("I assert that first heuristic is myHeuristic", "myHeuristic", pdCommandLine.getHeuristics()
                .get(0));
        assertEquals("I assert that second heuristic is myHeuristic2", "myHeuristic2", pdCommandLine.getHeuristics()
                .get(1));

    }

    public void testHeuristicArgs() {
        String[] args = new String[] { "-heur", "myHeuristic", "-cb", "codebase" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there is one heuristic", 1, pdCommandLine.getHeuristics().size());
        assertEquals("I assert that heuristic is myHeuristic", "myHeuristic", pdCommandLine.getHeuristics().get(0));
        args = new String[] { "-heuristics", "myHeuristic", "-cb", "codebase", "-heuristics", "myHeuristic2" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there are two heuristic", 2, pdCommandLine.getHeuristics().size());
        assertEquals("I assert that first heuristic is myHeuristic", "myHeuristic", pdCommandLine.getHeuristics()
                .get(0));
        assertEquals("I assert that second heuristic is myHeuristic2", "myHeuristic2", pdCommandLine.getHeuristics()
                .get(1));
        args = new String[] { "-heuristics", "myHeuristic", "myHeuristic2", "-cb", "codebase", "-heuristics",
                "myHeuristic3" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there are three heuristic", 3, pdCommandLine.getHeuristics().size());
        assertEquals("I assert that first heuristic is myHeuristic", "myHeuristic", pdCommandLine.getHeuristics()
                .get(0));
        assertEquals("I assert that second heuristic is myHeuristic2", "myHeuristic2", pdCommandLine.getHeuristics()
                .get(1));
        assertEquals("I assert that third heuristic is myHeuristic3", "myHeuristic3", pdCommandLine.getHeuristics()
                .get(2));

        args = new String[] { "-heur", "myHeuristic,myHeuristic2", "-cb", "codebase", "-heur", "myHeuristic3" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there are three heuristic", 3, pdCommandLine.getHeuristics().size());
        assertEquals("I assert that first heuristic is myHeuristic", "myHeuristic", pdCommandLine.getHeuristics()
                .get(0));
        assertEquals("I assert that second heuristic is myHeuristic2", "myHeuristic2", pdCommandLine.getHeuristics()
                .get(1));
        assertEquals("I assert that third heuristic is myHeuristic3", "myHeuristic3", pdCommandLine.getHeuristics()
                .get(2));

    }

    public void testEnginesArgs() {
        String[] args = new String[] { "-cb", "codebase", "-e", "myEngine" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there is one engine value", 1, pdCommandLine.getEngines().size());
        assertEquals("I assert that value of engine will be myEngine", "myEngine", pdCommandLine.getEngines().get(0));
        args = new String[] { "-cb", "codebase", "-engines", "myEngine", "-l", "123" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that there is one engine value", 1, pdCommandLine.getEngines().size());
        assertEquals("I assert that value of engine will be myEngine", "myEngine", pdCommandLine.getEngines().get(0));

    }

    public void testReportArg() {
        String[] args = new String[] { "-cb", "codebase", "-report", "rep.txt" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that report will be rep.txt", "rep.txt", pdCommandLine.getReport());
        args = new String[] { "-cb", "codebase", "-r", "rep.txt" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("I assert that report will be rep.txt", "rep.txt", pdCommandLine.getReport());

    }

    public void testHelpArg() {
        String[] args = new String[] { "-h" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertFalse("I assert that only help will be printed - that means that isAllArgumentsCorrect returns false",
                pdCommandLine.isAllArgumentsCorrect());

        args = new String[] { "--help" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertFalse("I assert that only help will be printed - that means that isAllArgumentsCorrect returns false",
                pdCommandLine.isAllArgumentsCorrect());

        args = new String[] { "-h", "-cb", "c:/src" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertFalse("I assert that only help will be printed - that means that isAllArgumentsCorrect returns false",
                pdCommandLine.isAllArgumentsCorrect());
    }

    public void testConfigurationFileParsingArg() {
        String[] args = new String[] { "-config", "src/test/resources/configuration/resources/config_test1.conf" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertTrue(
                "I assert that all arguments will be parsed from cconfig file - that means that isAllArgumentsCorrect returns true",
                pdCommandLine.isAllArgumentsCorrect());
        assertEquals("I assert that codebase is c:/src", "c:/src", pdCommandLine.getCodebase());
        assertEquals("report.txt", pdCommandLine.getReport());
        assertEquals(75, pdCommandLine.getLimit());
        assertEquals(2, pdCommandLine.getEngines().size());
        assertEquals("engine1", pdCommandLine.getEngines().get(0));
        assertEquals("engine2", pdCommandLine.getEngines().get(1));
        assertEquals("myheuristic", pdCommandLine.getHeuristics().get(0));
        assertEquals("java", pdCommandLine.getLanguage());
        assertTrue(pdCommandLine.isVerbose());
        assertEquals(5000, pdCommandLine.getWaitingTime());
        // it is not planed to use -h in config file but if we use it,it is
        // supposed to give the same results as if it used from command line
        args = new String[] { "-config", "src/test/resources/configuration/resources/config_test2.conf" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertFalse("I assert that only help will be printed - that means that isAllArgumentsCorrect returns false",
                pdCommandLine.isAllArgumentsCorrect());
        // config file can be written in single line
        args = new String[] { "-config", "src/test/resources/configuration/resources/config_test3.conf" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertTrue(
                "I assert that all arguments will be parsed from cconfig file - that means that isAllArgumentsCorrect returns true",
                pdCommandLine.isAllArgumentsCorrect());
        assertEquals("I assert that codebase is c:/src", "c:/src", pdCommandLine.getCodebase());
        assertEquals("report.txt", pdCommandLine.getReport());
        assertEquals(75, pdCommandLine.getLimit());
        assertEquals(2, pdCommandLine.getEngines().size());
        assertEquals("engine1", pdCommandLine.getEngines().get(0));
        assertEquals("engine2", pdCommandLine.getEngines().get(1));
        assertEquals("myheuristic", pdCommandLine.getHeuristics().get(0));
        // what if there is -config argument in config file
        // it will ignore -config argument in config file
        args = new String[] { "-config", "src/test/resources/configuration/resources/config_test4.conf" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertTrue(
                "I assert that all arguments will be parsed from cconfig file - that means that isAllArgumentsCorrect returns true",
                pdCommandLine.isAllArgumentsCorrect());
        assertEquals("I assert that codebase is c:/src1", "c:/src1", pdCommandLine.getCodebase());
        assertEquals(50, pdCommandLine.getLimit());
        assertEquals("I assert that waiting time will be set on minimal waiting time because it is not set in config",
                PdCommandLine.DEFAULT_WAITING_TIME, pdCommandLine.getWaitingTime());

    }

    public void testMallFormedConfigurationFileParsing() {
        String[] args = new String[] { "-config",
                "src/test/resources/configuration/resources/mallformed_config_test.conf" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertFalse("I assert that only help will be printed - that means that isAllArgumentsCorrect returns false",
                pdCommandLine.isAllArgumentsCorrect());

    }

    public void testVerboseParsing() {
        String[] args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine",
                "myEngine2", "-heur", "myHeuristic", "-language", "c++", "-verbose" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertTrue(pdCommandLine.isVerbose());

        args = new String[] { "-cb", "c:/src", "-report", "rep1.txt", "-v" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertTrue(pdCommandLine.isVerbose());
    }

    public void testLanguageParsing() {
        String[] args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine",
                "myEngine2", "-heur", "myHeuristic", "-language", "c++", "-verbose" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("c++", pdCommandLine.getLanguage());

        args = new String[] { "-cb", "c:/src", "-language", "c++" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("c++", pdCommandLine.getLanguage());
    }

    public void testGetReportFormatParsing() {
        String[] args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine",
                "myEngine2", "-heur", "myHeuristic", "-language", "c++", "-verbose" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("txt", pdCommandLine.getReportFormat());

        args = new String[] { "-cb", "c:/src", "-report", "rep.html", "-limit", "23", "-e", "myEngine", "myEngine2",
                "-heur", "myHeuristic", "-language", "c++", "-verbose" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals("html", pdCommandLine.getReportFormat());

    }

    public void testWaitingTimeParsing() {
        String[] args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine",
                "myEngine2", "-heur", "myHeuristic", "-language", "c++", "-verbose", "-w", "1200" };
        PdCommandLine pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals(1200, pdCommandLine.getWaitingTime());
        args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine", "myEngine2",
                "-heur", "myHeuristic", "-language", "c++", "-verbose", "-wait", "1234" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals(1234, pdCommandLine.getWaitingTime());
        args = new String[] { "-cb", "c:/src", "-report", "rep.txt", "-limit", "23", "-e", "myEngine", "myEngine2",
                "-heur", "myHeuristic", "-language", "c++", "-verbose", "-wait", "1500" };
        pdCommandLine = new PdCommandLine(args, System.out);
        assertEquals(1500, pdCommandLine.getWaitingTime());

    }
}
