/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.core;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import junit.framework.TestCase;

import org.apache.rat.pd.engines.ISearchEngine;
import org.apache.rat.pd.engines.SearchResult;
import org.apache.rat.pd.heuristic.BruteForceHeuristicChecker;
import org.apache.rat.pd.heuristic.HeuristicCheckerResult;
import org.apache.rat.pd.heuristic.IHeuristicChecker;
import org.apache.rat.pd.report.Report;

/**
 * @author maka: this is test for sliding window algorithm:
 *         http://wiki.apache.org/general/SummerOfCode2009#rat-project T is
 *         token which have to be found by search engine . F is token which
 *         search engine will not found.
 */
public class SlidingWindowAlgorithmTest extends TestCase {

    private List<ISearchEngine> searchEngines;

    private SourceCodeAnalyser analyser;

    private ReportDocumentMock reportDocument;

    private List<IHeuristicChecker> checkers;

    protected void setUp() throws Exception {
        super.setUp();
        reportDocument = new ReportDocumentMock();
        searchEngines = new ArrayList<ISearchEngine>();
        searchEngines.add(new MySearchEngineMock());
        checkers = new ArrayList<IHeuristicChecker>();

        analyser = new SourceCodeAnalyser(searchEngines, checkers, reportDocument, System.out);

    }

    public void testSlidingWindowAlgorithm() {
        checkers.add(new BruteForceHeuristicChecker("lang"));

        analyser.checkOneFile(searchEngines, "TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T TEND F", checkers, "");
        assertEquals("there must be 1 chunk of code found with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T TEND F TEND", checkers, "");
        assertEquals("there must be 2 chunks of code found with search engine", 2, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T TEND   F F F    TEND    F F F", checkers, "");
        assertEquals("there must be 2 chunks of code found with search engine", 2, reportDocument.size());

        reportDocument.clear();
        analyser.checkOneFile(searchEngines, "F T TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found with search engine", 1, reportDocument.size());

        reportDocument.clear();
        analyser.checkOneFile(searchEngines, "F", checkers, "");
        assertEquals("there must be 0 chunks of code found with search engine", 0, reportDocument.size());

        reportDocument.clear();
        analyser.checkOneFile(searchEngines, "F F F", checkers, "");
        assertEquals("there must be 0 chunks of code found with search engine", 0, reportDocument.size());

        // And, of course blank string :)
        reportDocument.clear();
        analyser.checkOneFile(searchEngines, "", checkers, "");
        assertEquals("there must be 0 chunks of code found with search engine", 0, reportDocument.size());

    }

    public void testSlidingWindowAlgorithmWithHeuristicCheckeer() {
        checkers.add(new MyHeuristicCheckerMock());

        analyser.checkOneFile(searchEngines, "T", checkers, "");
        assertEquals("there must be 0 chunk of code found  with search engine", 0, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "TEND TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 2, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T TEND T T TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 2, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F T", checkers, "");
        assertEquals("there must be 0 chunk of code found  with search engine", 0, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T T", checkers, "");
        assertEquals("there must be 0 chunk of code found  with search engine", 0, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T T TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F F", checkers, "");
        assertEquals("there must be 0 chunk of code found  with search engine", 0, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F F TEND", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F F TEND F", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F F TEND T", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "F F TEND TEND", checkers, "");
        assertEquals("there must be 2 chunk of code found  with search engine", 2, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "TEND F", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

        analyser.checkOneFile(searchEngines, "T TEND F F F", checkers, "");
        assertEquals("there must be 1 chunk of code found  with search engine", 1, reportDocument.size());

        reportDocument.clear();

    }

    /**
     * for testing
     * 
     * @author maka
     */
    private static class MySearchEngineMock implements ISearchEngine {

        @Override
        public List<SearchResult> getSearchResults() {
            List<SearchResult> toRet = new ArrayList<SearchResult>();
            toRet.add(new SearchResult());
            return toRet;
        }

        @Override
        public boolean isCodeFound(String possibleCopyAndPastedCode) {
            String temp = possibleCopyAndPastedCode.replaceAll("(F )*(T )*TEND", "");
            if (temp.trim().length() > 0) {
                return false;
            } else
                return true;
        }
    }

    /**
     * for testing
     * 
     * @author maka:
     */
    private static class MyHeuristicCheckerMock implements IHeuristicChecker {
        @Override
        public HeuristicCheckerResult checkByHeuristic(String codeToBeChecked) {
            HeuristicCheckerResult toret = new HeuristicCheckerResult(false, true, codeToBeChecked,
                    MyHeuristicCheckerMock.class);

            // Create a pattern to match comments
            Pattern p = Pattern.compile("(F )*(T )*TEND", Pattern.MULTILINE);
            // Run matches
            Matcher m = p.matcher(codeToBeChecked);
            // there must be no characters after comment
            if (m.find()) {
                String found = m.group();
                if (codeToBeChecked.equals(found)) {
                    toret = new HeuristicCheckerResult(true, false, found, null);
                    System.out.println("Found comment: " + found);
                }
            }
            return toret;

        }

        @Override
        public String getLanguage() {
            return "lang";
        }
    };

    /**
     * for testing, some methods are added to enable testing
     * 
     * @author maka
     */
    private static class ReportDocumentMock extends Report {

        public void clear() {
            super.raportList.clear();
        }

        public int size() {
            return super.raportList.size();
        }

        @Override
        public String getReport() {
            return "";
        }
    }

}
