/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.functions;

import org.apache.rat.pd.heuristic.HeuristicCheckerResult;

import junit.framework.TestCase;

public class CPPFunctionHeuristicCheckerTest extends TestCase {
    private CPPFunctionHeuristicChecker checker;

    private static final String SIMPLE_FUNCTION_1 = "void * name (){}";
    private static final String SIMPLE_FUNCTION_2 = "public void name  (int prom1  , \n string prom 2) {\n something; \n}";
    private static final String SIMPLE_FUNCTION_3 = "void BubbleSort(apvector <int> &num)\n" + "{\n"
            + "      int i, j, flag = 1;    // set flag to 1 to start first pass\n"
            + "     int temp;             // holding variable\n" + "    int numLength = num.length( ); \n"
            + "    for(i = 1; (i <= numLength) && flag; i++)\n" + "   {\n" + "        flag = 0;\n"
            + "        for (j=0; j < (numLength -1); j++)\n" + "       {\n"
            + "              if (num[j+1] > num[j])      // ascending order simply changes to <\n"
            + "              { \n" + "                   temp = num[j];             // swap elements\n"
            + "                   num[j] = num[j+1];\n" + "                  num[j+1] = temp;\n"
            + "                  flag = 1;               // indicates that a swap occurred.\n" + "              }\n"
            + "          }\n" + "     }\n"
            + "     return;   //arrays are passed to functions by address; nothing is returned\n" + "}";

    private static final String SIMPLE_FUNCTION_4 = "MyClass::~ MyClass(MyClass&)\n" + "{\n"
            + "    std::cout << \"Simple Cat Constructor...\n\";\n" + "    itsAge = 1;\n" + "}";
    private static final String SIMPLE_FUNCTION_5 = "MyType MyType::Add(const MyType & rhs)\n" + " {\n"
            + "     return MyType(itsVal+ rhs.GetItsVal());\n" + " }";

    private static final String SIMPLE_MACRO_FUNCTION_1 = "#define vecswap(a, \nb, n)        if ((n) > 0) swapfunc((char *)a, (char *)b, (int)n, swaptype)\n";
    private static final String SIMPLE_MACRO_FUNCTION_2 = "#define vecswap(a, b, n) if ((n) > 0) swapfunc((char *)a, (char *)b, (int)n, swaptype)\n";
    private static final String SIMPLE_MACRO_FUNCTION_3 = "#define maxOfTwoParametrs(a,b) \n"
            + "       {\n typeof (a) _a = (a); \n" + "           typeof (b) _b = (b); \n"
            + "         _a > _b ? _a : _b; }";
    private static final String SIMPLE_MACRO_FUNCTION_4 = "#define VFTDELTA_VAL()      0,\n";
    private static final String SIMPLE_MACRO_FUNCTION_5 = "#define swap_code(TYPE*, parmi$, parmj, n) {               \n"
            + "        long i = (n) / sizeof (TYPE);                   \n"
            + "        register TYPE *pi = (TYPE *) (parmi);           \n"
            + "        register TYPE *pj = (TYPE *) (parmj);           \n"
            + "        do {                                            \n"
            + "                register TYPE   t = *pi;                \n"
            + "                *pi++ = *pj;                            \n"
            + "                *pj++ = t;                             \n"
            + "        } while (--i > 0);                              \n" + "}";

    private static final String SIMPLE_INLINE_FUNCTION_1 = "inline PRInt32 MinInt(PRInt32 x, PRInt32 y)\n" + " {\n"
            + "     return NS_MIN(x, y);\n" + "  }";

    private static final String SIMPLE_INLINE_FUNCTION_2 = "inline PRInt32 Min_Int(PRInt32 x, PRInt32 y)\n" + " {\n"
            + "     return NS_MIN(x, y);\n" + "  }";

    private static final String SIMPLE_INLINE_FUNCTION_3 = "inline void\n" + "glMultMatrix( const Imath::M44f* m )\n"
            + "{\n" + "    throwBadMatrix (*m);\n" + "    glMultMatrixf( (GLfloat*)(*m)[0] );\n" + "}";

    private static final String SIMPLE_INLINE_FUNCTION_4 = "inline\n"
            + "size_t SearchResultMessageArg::nTried() const\n" + "{\n" + "  return filename_.size();\n" + "}";

    private static final String SIMPLE_INLINE_FUNCTION_5 = "inline void A::UpdateX(int newX)\n" + "	\n" + "{\n"
            + "		\n" + "  if (g_y != 0 && m_x < newX)\n" + "  \n" + "  {\n" + "	  \n" + "    m_x = newX;\n" + "    \n"
            + "  }\n" + "  \n" + "}";

    private static final String INVALID_FUNCTION_1 = "int sum(){}\n something";
    private static final String INVALID_FUNCTION_2 = "void name(int pom)\n{\nsomething{}";

    private static final String SIMPLE_MACRO_FUNCTION_6 = "something #define name(){}";
    private static final String SIMPLE_MACRO_FUNCTION_6_SUGGESTED = "#define name(){}";
    private static final String INVALID_MACRO_FUNCTION_2 = "#define name() {something {}";

    private static final String INVALID_INLINE_FUNCTION_1 = "inline void name() { {}";

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        checker = new CPPFunctionHeuristicChecker(0, System.out);
    }

    /**
     * Test method for
     * {@link org.apache.rat.pd.heuristic.functions.FunctionHeuristicChecker#checkByHeuristic(java.lang.String)}
     * .
     */
    public void testCheckByHeuristic() {
        assertTrue("This is a function", checker.checkByHeuristic(SIMPLE_FUNCTION_1).isCheckOnInternetNeaded());
        assertTrue("This is a function", checker.checkByHeuristic(SIMPLE_FUNCTION_2).isCheckOnInternetNeaded());
        assertTrue("This is a function for bubble sort- see http://mathbits.com/mathbits/compsci/Arrays/Bubble.htm",
                checker.checkByHeuristic(SIMPLE_FUNCTION_3).isCheckOnInternetNeaded());

        assertTrue("This is a function for bubble sort- see http://mathbits.com/mathbits/compsci/Arrays/Bubble.htm",
                checker.checkByHeuristic(SIMPLE_FUNCTION_4).isCheckOnInternetNeaded());

        assertTrue("This is a function", checker.checkByHeuristic(SIMPLE_FUNCTION_5).isCheckOnInternetNeaded());

        assertFalse("There must be false return because there is something after function", checker.checkByHeuristic(
                INVALID_FUNCTION_1).isCheckOnInternetNeaded());
        assertFalse(
                "There must be false return because the numbers of { and } are not equal so this function is not entire",
                checker.checkByHeuristic(INVALID_FUNCTION_2).isCheckOnInternetNeaded());

    }

    public void testByHeuristicCheckerMacro() {

        assertTrue("This is valid macro", checker.checkByHeuristic(SIMPLE_MACRO_FUNCTION_1).isCheckOnInternetNeaded());
        assertTrue("This is a valid macro which have to many parametrs to write them in one line", checker
                .checkByHeuristic(SIMPLE_MACRO_FUNCTION_2).isCheckOnInternetNeaded());
        assertTrue("This is a valid macro whish is multiline", checker.checkByHeuristic(SIMPLE_MACRO_FUNCTION_3)
                .isCheckOnInternetNeaded());
        assertTrue("This is a valid macro wwhish is multiline", checker.checkByHeuristic(SIMPLE_MACRO_FUNCTION_4)
                .isCheckOnInternetNeaded());
        assertTrue("This is a valid macro whish is multiline", checker.checkByHeuristic(SIMPLE_MACRO_FUNCTION_5)
                .isCheckOnInternetNeaded());
        HeuristicCheckerResult result = checker.checkByHeuristic(SIMPLE_MACRO_FUNCTION_6);
        assertTrue("This string consist macro", result.isCheckOnInternetNeaded());
        assertEquals("Suggested macro must look like this:", SIMPLE_MACRO_FUNCTION_6_SUGGESTED, result
                .getCodeSuggestedToBeChecked());
        assertFalse("There must be false returned because the numbers of open and closed brackets are not equal",
                checker.checkByHeuristic(INVALID_MACRO_FUNCTION_2).isCheckOnInternetNeaded());

    }

    public void testByHeuristicCheckerInlineFunctions() {

        assertTrue("This is a valid multiline inline function", checker.checkByHeuristic(SIMPLE_INLINE_FUNCTION_1)
                .isCheckOnInternetNeaded());
        assertTrue("This is a valid multiline inline function", checker.checkByHeuristic(SIMPLE_INLINE_FUNCTION_2)
                .isCheckOnInternetNeaded());
        assertTrue("This is a valid multiline inline function", checker.checkByHeuristic(SIMPLE_INLINE_FUNCTION_3)
                .isCheckOnInternetNeaded());
        assertTrue("This is a valid multiline inline function", checker.checkByHeuristic(SIMPLE_INLINE_FUNCTION_4)
                .isCheckOnInternetNeaded());
        assertTrue("This is a valid multiline inline function", checker.checkByHeuristic(SIMPLE_INLINE_FUNCTION_5)
                .isCheckOnInternetNeaded());

        assertFalse("There must be false returned because the numbers of open and closed brackets are not equal",
                checker.checkByHeuristic(INVALID_INLINE_FUNCTION_1).isCheckOnInternetNeaded());

    }

}
