/*
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 */
package org.apache.rat.pd.heuristic.misspellings;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

public class CodeHeuristicsSpellCheckerTest extends TestCase {

    private final static String CORRECT_CODE_1 = "public static void main (String[] arguments)";
    private final static String[] CORRECT_CODE_1_EXPECTED = { "public", "static", "void", "main", "String", "arguments" };

    private final static String CORRECT_CODE_2 = "public static void main (String[] arguments){\n"
            + "System.out.print(\"something\");\n" + "}";
    private final static String[] CORRECT_CODE_2_EXPECTED = { "public", "static", "void", "main", "String",
            "arguments", "System", "out", "print", "something" };

    private final static String CORRECT_CODE_3 = "for (i=0;i<count;i++){run();}";
    private final static String[] CORRECT_CODE_3_EXPECTED = { "for", "i", "i", "count", "i", "run" };

    private final static String[] CAMEL_CASE_STRING_1 = { "thisIsMyName" };
    private final static String[] CAMEL_CASE_STRING_1_EXPECTED = { "this", "Is", "My", "Name" };

    private final static String[] CAMEL_CASE_STRING_2 = { "BigNumber" };
    private final static String[] CAMEL_CASE_STRING_2_EXPECTED = { "Big", "Number" };

    private final static String[] UNDERSCORE_COMBINED_STRING_1 = { "this_is_my_name" };
    private final static String[] UNDERSCORE_COMBINED_STRING_1_EXPECTED = { "this", "is", "my", "name" };

    private final static String[] UNDERSCORE_COMBINED_STRING_2 = { "_variable" };
    private final static String[] UNDERSCORE_COMBINED_STRING_2_EXPECTED = { "variable" };

    private final static String[] UNDERSCORE_COMBINED_STRING_3 = { "_variable_" };
    private final static String[] UNDERSCORE_COMBINED_STRING_3_EXPECTED = { "variable" };

    private final static String[] UNDERSCORE_COMBINED_STRING_4 = { "__interesting__variable__" };
    private final static String[] UNDERSCORE_COMBINED_STRING_4_EXPECTED = { "interesting", "variable" };

    private final static String APACHE_LICENCE = "Licensed to the Apache Software Foundation (ASF) under one"
            + " or more contributor license agreements.  See the NOTICE file"
            + " distributed with this work for additional information"
            + " regarding copyright ownership.  The ASF licenses this file"
            + " to you under the Apache License, Version 2.0 (the"
            + " \"License\"); you may not use this file except in compliance"
            + " with the License.  You may obtain a copy of the License at" + ""
            + "   http://www.apache.org/licenses/LICENSE-2.0" + ""
            + " Unless required by applicable law or agreed to in writing,"
            + " software distributed under the License is distributed on an"
            + " \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY"
            + " KIND, either express or implied.  See the License for the"
            + " specific language governing permissions and limitations" + " under the License.";

    protected MisspellingsHeuristicChecker checker = null;

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        InputStream is = getClass().getResourceAsStream("/misspellings/english.dict");
        IDictionary dictionary = new Dictionary(is);
        checker = new MisspellingsHeuristicChecker(dictionary, "lang", System.out);
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testCheckByHeuristic() {
        assertFalse(checker.checkByHeuristic("book").isCheckOnInternetNeaded());
        assertTrue(checker.checkByHeuristic("abcdef").isCheckOnInternetNeaded());
        assertTrue(checker.checkByHeuristic("bookc").isCheckOnInternetNeaded());
        assertTrue(checker.checkByHeuristic("bookc book").isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic("book book").isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic("book_book").isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic("book_book book").isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic(CORRECT_CODE_1).isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic("public static void main (String[] arguments) { " + "return 0" + "}")
                .isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic("bookBookLook").isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic("CodeHeuristicSpellChecker").isCheckOnInternetNeaded());
        assertFalse(checker.checkByHeuristic(APACHE_LICENCE).isCheckOnInternetNeaded());
    }

    public void testisWordCorrect() throws IOException {
        assertTrue(checker.isWordCorrect(new ArrayList<String>(Arrays.asList(new String[] { "is" }))));
        assertTrue(checker.isWordCorrect(new ArrayList<String>(Arrays.asList(new String[] { "Is" }))));
        assertTrue(checker.isWordCorrect(new ArrayList<String>(Arrays.asList(new String[] { "Apache" }))));

        assertFalse(checker.isWordCorrect(new ArrayList<String>(Arrays.asList(new String[] { "mispelled" }))));
    }

    public void testFilterWordsFromCode() {
        List<String> actual;
        List<String> expected;
        actual = checker.filterWordsFromCode(CORRECT_CODE_1);
        expected = new ArrayList<String>(Arrays.asList(CORRECT_CODE_1_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

        actual = checker.filterWordsFromCode(CORRECT_CODE_2);
        expected = new ArrayList<String>(Arrays.asList(CORRECT_CODE_2_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

        actual = checker.filterWordsFromCode(CORRECT_CODE_3);
        expected = new ArrayList<String>(Arrays.asList(CORRECT_CODE_3_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);
    }

    public void testDecomposeCamelCombinedWords() {

        List<String> actual;
        List<String> expected;
        actual = checker.decomposeCamelCombinedWords(new ArrayList<String>(Arrays.asList(CAMEL_CASE_STRING_1)));
        expected = new ArrayList<String>(Arrays.asList(CAMEL_CASE_STRING_1_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

        actual = checker.decomposeCamelCombinedWords(new ArrayList<String>(Arrays.asList(CAMEL_CASE_STRING_2)));
        expected = new ArrayList<String>(Arrays.asList(CAMEL_CASE_STRING_2_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

    }

    public void testDecomposeUnderscoreCombinedWords() {

        List<String> actual;
        List<String> expected;
        actual = checker.decomposeUnderscoreCombinedWords(new ArrayList<String>(Arrays
                .asList(UNDERSCORE_COMBINED_STRING_1)));
        expected = new ArrayList<String>(Arrays.asList(UNDERSCORE_COMBINED_STRING_1_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

        actual = checker.decomposeUnderscoreCombinedWords(new ArrayList<String>(Arrays
                .asList(UNDERSCORE_COMBINED_STRING_2)));
        expected = new ArrayList<String>(Arrays.asList(UNDERSCORE_COMBINED_STRING_2_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

        actual = checker.decomposeUnderscoreCombinedWords(new ArrayList<String>(Arrays
                .asList(UNDERSCORE_COMBINED_STRING_3)));
        expected = new ArrayList<String>(Arrays.asList(UNDERSCORE_COMBINED_STRING_3_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

        actual = checker.decomposeUnderscoreCombinedWords(new ArrayList<String>(Arrays
                .asList(UNDERSCORE_COMBINED_STRING_4)));
        expected = new ArrayList<String>(Arrays.asList(UNDERSCORE_COMBINED_STRING_4_EXPECTED));
        assertEquals("This two lists must be equals", expected, actual);

    }
}
